=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2011-12-21 00:07:49 +0000
+++ bzrlib/dirstate.py	2012-01-05 11:16:45 +0000
@@ -1924,7 +1924,7 @@
             # paths are produced by UnicodeDirReader on purpose.
             abspath = abspath.encode(fs_encoding)
         target = os.readlink(abspath)
-        if fs_encoding not in ('UTF-8', 'US-ASCII', 'ANSI_X3.4-1968'):
+        if fs_encoding not in ('utf-8', 'ascii'):
             # Change encoding if needed
             target = target.decode(fs_encoding).encode('UTF-8')
         return target

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2012-01-05 15:54:17 +0000
+++ bzrlib/tests/blackbox/test_add.py	2012-01-05 17:56:30 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -154,8 +154,8 @@
         new_tree = self.make_branch_and_tree('new')
         self.build_tree(['new/a', 'new/b/', 'new/b/c', 'd'])
 
-        os.chdir('new')
-        out, err = self.run_bzr('add --file-ids-from ../base')
+        out, err = self.run_bzr('add --file-ids-from ../base',
+                                working_dir='new')
         self.assertEqual('', err)
         self.assertEqualDiff('adding a w/ file id from a\n'
                              'adding b w/ file id from b\n'
@@ -175,14 +175,14 @@
         new_tree = self.make_branch_and_tree('new')
         self.build_tree(['new/c', 'new/d'])
 
-        os.chdir('new')
-        out, err = self.run_bzr('add --file-ids-from ../base/b')
+        out, err = self.run_bzr('add --file-ids-from ../base/b',
+                                working_dir='new')
         self.assertEqual('', err)
         self.assertEqualDiff('adding c w/ file id from b/c\n'
                              'adding d w/ file id from b/d\n',
                              out)
 
-        new_tree = new_tree.bzrdir.open_workingtree()
+        new_tree = new_tree.bzrdir.open_workingtree('new')
         self.assertEqual(base_tree.path2id('b/c'), new_tree.path2id('c'))
         self.assertEqual(base_tree.path2id('b/d'), new_tree.path2id('d'))
 

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2012-01-05 14:26:58 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2012-01-05 17:56:30 +0000
@@ -17,14 +17,12 @@
 
 """Tests of bound branches (binding, unbinding, commit, etc) command."""
 
-import os
-
 from bzrlib import (
+    branch,
+    bzrdir,
     errors,
     tests,
     )
-from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
 from bzrlib.tests import script
 
 
@@ -42,27 +40,26 @@
         child_tree = branch.create_checkout('child')
 
         self.check_revno(1, 'child')
-        d = BzrDir.open('child')
+        d = bzrdir.BzrDir.open('child')
         self.assertNotEqual(None, d.open_branch().get_master_branch())
 
         return base_tree, child_tree
 
     def check_revno(self, val, loc='.'):
         self.assertEqual(
-            val, BzrDir.open(loc).open_branch().last_revision_info()[0])
+            val, bzrdir.BzrDir.open(loc).open_branch().last_revision_info()[0])
 
     def test_simple_binding(self):
         tree = self.make_branch_and_tree('base')
         self.build_tree(['base/a', 'base/b'])
         tree.add('a', 'b')
         tree.commit(message='init')
-        branch = tree.branch
 
         tree.bzrdir.sprout('child')
 
         self.run_bzr('bind ../base', working_dir='child')
 
-        d = BzrDir.open('child')
+        d = bzrdir.BzrDir.open('child')
         self.assertNotEqual(None, d.open_branch().get_master_branch())
 
         self.run_bzr('unbind', working_dir='child')
@@ -73,7 +70,8 @@
     def test_bind_branch6(self):
         branch1 = self.make_branch('branch1', format='dirstate-tags')
         error = self.run_bzr('bind', retcode=3, working_dir='branch1')[1]
-        self.assertContainsRe(error, 'no previous location known')
+        self.assertEndsWith(
+            error, 'No location supplied and no previous location known\n')
 
     def setup_rebind(self, format):
         branch1 = self.make_branch('branch1')
@@ -84,13 +82,15 @@
     def test_rebind_branch6(self):
         self.setup_rebind('dirstate-tags')
         self.run_bzr('bind', working_dir='branch2')
-        b = Branch.open('branch2')
-        self.assertContainsRe(b.get_bound_location(), '\/branch1\/$')
+        b = branch.Branch.open('branch2')
+        self.assertEndsWith(b.get_bound_location(), '/branch1/')
 
     def test_rebind_branch5(self):
         self.setup_rebind('knit')
         error = self.run_bzr('bind', retcode=3, working_dir='branch2')[1]
-        self.assertContainsRe(error, 'old locations')
+        self.assertEndsWith(
+            error, 'No location supplied.  This format does not remember'
+            ' old locations.\n')
 
     def test_bound_commit(self):
         child_tree = self.create_branches()[1]
@@ -181,7 +181,6 @@
         # the bound parent branch
         self.run_bzr('pull ../newchild', working_dir='child')
         self.check_revno(2, 'child')
-
         self.check_revno(2, 'base')
 
     def test_pull_local_updates_local(self):
@@ -195,7 +194,6 @@
         # the bound parent branch
         self.run_bzr('pull ../newchild --local', working_dir='child')
         self.check_revno(2, 'child')
-
         self.check_revno(1, 'base')
 
     def test_bind_diverged(self):
@@ -221,10 +219,8 @@
         child_tree.update()
         child_tree.commit(message='merged')
         self.check_revno(3, 'child')
-
-        self.assertEquals(
-            child_tree.branch.last_revision(),
-            base_tree.branch.last_revision())
+        self.assertEquals(child_tree.branch.last_revision(),
+                          base_tree.branch.last_revision())
 
     def test_bind_parent_ahead(self):
         base_tree = self.create_branches()[0]
@@ -286,35 +282,6 @@
                            working_dir='tree_1')
         self.assertIs(None, tree.branch.get_bound_location())
 
-    def test_bind_nick(self):
-        """Bind should not update implicit nick."""
-        base = self.make_branch_and_tree('base')
-        child = self.make_branch_and_tree('child')
-        self.assertEqual(child.branch.nick, 'child')
-        self.assertEqual(False,
-                         child.branch.get_config().has_explicit_nickname())
-        self.run_bzr('bind ../base', working_dir='child')
-        # Refresh the child tree/branch objects as 'bind' modified them
-        child = BzrDir.open('child').open_workingtree()
-        self.assertEqual(child.branch.nick, base.branch.nick)
-        self.assertEqual(False,
-                         child.branch.get_config().has_explicit_nickname())
-
-    def test_bind_explicit_nick(self):
-        """Bind should update explicit nick."""
-        base = self.make_branch_and_tree('base')
-        child = self.make_branch_and_tree('child')
-        child.branch.nick = "explicit_nick"
-        self.assertEqual("explicit_nick", child.branch.nick)
-        self.assertEqual("explicit_nick",
-                         child.branch.get_config()._get_explicit_nickname())
-        self.run_bzr('bind ../base', working_dir='child')
-        # Refresh the child tree/branch objects as 'bind' modified them
-        child = BzrDir.open('child').open_workingtree()
-        self.assertEqual(child.branch.nick, base.branch.nick)
-        self.assertEqual(child.branch.get_config()._get_explicit_nickname(),
-                         base.branch.nick)
-
     def test_commit_after_merge(self):
         base_tree, child_tree = self.create_branches()
 
@@ -344,11 +311,8 @@
         # Commit should succeed, and cause merged revisions to
         # be pulled into base
         self.run_bzr(['commit', '-m', 'merge other'], working_dir='child')
-
         self.check_revno(2, 'child')
-
         self.check_revno(2, 'base')
-
         self.assertTrue(base_tree.branch.repository.has_revision(new_rev_id))
 
     def test_pull_overwrite(self):
@@ -390,7 +354,7 @@
         branch = tree.branch
         tree.bzrdir.sprout('child')
         self.run_bzr('bind --directory=child base')
-        d = BzrDir.open('child')
+        d = bzrdir.BzrDir.open('child')
         self.assertNotEqual(None, d.open_branch().get_master_branch())
         self.run_bzr('unbind -d child')
         self.assertEqual(None, d.open_branch().get_master_branch())

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2012-01-05 15:54:17 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2012-01-05 17:56:30 +0000
@@ -25,9 +25,9 @@
     controldir,
     errors,
     revision as _mod_revision,
+    tests,
     )
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
-from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests import (
     fixtures,
     test_server,
@@ -43,7 +43,7 @@
 from bzrlib.workingtree import WorkingTree
 
 
-class TestBranch(TestCaseWithTransport):
+class TestBranch(tests.TestCaseWithTransport):
 
     def example_branch(self, path='.', format=None):
         tree = self.make_branch_and_tree(path, format=format)
@@ -83,7 +83,8 @@
         self.assertEqual('', err)
         out,err = self.run_bzr('branch a file:b,branch=orig', retcode=3)
         self.assertEqual('', out)
-        self.assertEqual('bzr: ERROR: Already a branch: "file:b,branch=orig".\n', err)
+        self.assertEqual(
+            'bzr: ERROR: Already a branch: "file:b,branch=orig".\n', err)
 
     def test_from_colocated(self):
         """Branch from a colocated branch into a regular branch."""
@@ -113,7 +114,7 @@
             else:
                 corrupt = '\xFF'
             f.write(corrupt) # make sure we corrupt something
-        self.run_bzr_error(['Corruption while decompressing repository file'], 
+        self.run_bzr_error(['Corruption while decompressing repository file'],
                             'branch a b', retcode=3)
 
     def test_branch_switch_no_branch(self):
@@ -175,7 +176,8 @@
         #     the new branch
         self.example_branch('a')
         self.run_bzr('checkout --lightweight a current')
-        out, err = self.run_bzr('branch --switch ../a ../b', working_dir='current')
+        out, err = self.run_bzr('branch --switch ../a ../b',
+                                working_dir='current')
         a = branch.Branch.open('a')
         b = branch.Branch.open('b')
         self.assertEqual(a.last_revision(), b.last_revision())
@@ -345,16 +347,16 @@
         new_branch.repository.get_revision('rev-2')
 
 
-class TestBranchStacked(TestCaseWithTransport):
+class TestBranchStacked(tests.TestCaseWithTransport):
     """Tests for branch --stacked"""
 
     def assertRevisionInRepository(self, repo_path, revid):
-        """Check that a revision is in a repository, disregarding stacking."""
+        """Check that a revision is in a repo, disregarding stacking."""
         repo = bzrdir.BzrDir.open(repo_path).open_repository()
         self.assertTrue(repo.has_revision(revid))
 
     def assertRevisionNotInRepository(self, repo_path, revid):
-        """Check that a revision is not in a repository, disregarding stacking."""
+        """Check that a revision is not in a repo, disregarding stacking."""
         repo = bzrdir.BzrDir.open(repo_path).open_repository()
         self.assertFalse(repo.has_revision(revid))
 
@@ -432,7 +434,8 @@
             trunk_tree.branch.base, err)
         self.assertRevisionNotInRepository('newbranch', original_revid)
         new_branch = branch.Branch.open('newbranch')
-        self.assertEqual(trunk_tree.branch.base, new_branch.get_stacked_on_url())
+        self.assertEqual(trunk_tree.branch.base,
+                         new_branch.get_stacked_on_url())
 
     def test_branch_stacked_from_smart_server(self):
         # We can branch stacking on a smart server
@@ -473,7 +476,7 @@
             err)
 
 
-class TestSmartServerBranching(TestCaseWithTransport):
+class TestSmartServerBranching(tests.TestCaseWithTransport):
 
     def test_branch_from_trivial_branch_to_same_server_branch_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -598,10 +601,11 @@
         self.assertFalse(t.has('remote/file'))
 
 
-class TestDeprecatedAliases(TestCaseWithTransport):
+class TestDeprecatedAliases(tests.TestCaseWithTransport):
 
     def test_deprecated_aliases(self):
-        """bzr branch can be called clone or get, but those names are deprecated.
+        """bzr branch can be called clone or get, but those names are
+        deprecated.
 
         See bug 506265.
         """

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2011-12-14 21:57:37 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2011 Canonical Ltd
+# Copyright (C) 2005-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
 """Black-box tests for bzr cat.
 """
 
-import os
-
 from bzrlib import tests
 from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.transport import memory
@@ -31,45 +29,45 @@
         tree = self.make_branch_and_tree('branch')
         self.build_tree_contents([('branch/a', 'foo\n')])
         tree.add('a')
-        os.chdir('branch')
         # 'bzr cat' without an option should cat the last revision
-        self.run_bzr(['cat', 'a'], retcode=3)
+        self.run_bzr(['cat', 'a'], retcode=3, working_dir='branch')
 
         tree.commit(message='1')
-        self.build_tree_contents([('a', 'baz\n')])
+        self.build_tree_contents([('branch/a', 'baz\n')])
 
-        self.assertEquals(self.run_bzr(['cat', 'a'])[0], 'foo\n')
+        self.assertEquals('foo\n',
+                          self.run_bzr(['cat', 'a'], working_dir='branch')[0])
 
         # On Windows, we used to have a bug where newlines got changed into
         # crlf, whereas cat ought to write out the file exactly as it's
         # recorded (by default.)  That problem can't be reproduced in-process,
         # so we need just one test here that 
-        self.assertEquals(self.run_bzr_subprocess(['cat', 'a'])[0], 'foo\n')
+        self.assertEquals('foo\n',
+                          self.run_bzr_subprocess(['cat', 'a'],
+                                                  working_dir='branch')[0])
 
         tree.commit(message='2')
-        self.assertEquals(self.run_bzr(['cat', 'a'])[0], 'baz\n')
-        self.assertEquals(self.run_bzr(
-            ['cat', 'a', '-r', '1'])[0],
-            'foo\n')
-        self.assertEquals(self.run_bzr(
-            ['cat', 'a', '-r', '-1'])[0],
-            'baz\n')
+        self.assertEquals(
+            'baz\n', self.run_bzr(['cat', 'a'], working_dir='branch')[0])
+        self.assertEquals(
+            'foo\n', self.run_bzr(['cat', 'a', '-r', '1'],
+                                  working_dir='branch')[0])
+        self.assertEquals(
+            'baz\n', self.run_bzr(['cat', 'a', '-r', '-1'],
+                                  working_dir='branch')[0])
 
         rev_id = tree.branch.last_revision()
 
-        self.assertEquals(self.run_bzr(
-            ['cat', 'a', '-r', 'revid:%s' % rev_id])[0],
-            'baz\n')
-
-        os.chdir('..')
-
-        self.assertEquals(self.run_bzr(
-            ['cat', 'branch/a', '-r', 'revno:1:branch'])[0],
-            'foo\n')
+        self.assertEquals(
+            'baz\n', self.run_bzr(['cat', 'a', '-r', 'revid:%s' % rev_id],
+                                  working_dir='branch')[0])
+
+        self.assertEquals('foo\n',
+                          self.run_bzr(['cat', 'branch/a',
+                                        '-r', 'revno:1:branch'])[0])
         self.run_bzr(['cat', 'a'], retcode=3)
-        self.run_bzr(
-                ['cat', 'a', '-r', 'revno:1:branch-that-does-not-exist'],
-                retcode=3)
+        self.run_bzr(['cat', 'a', '-r', 'revno:1:branch-that-does-not-exist'],
+                     retcode=3)
 
     def test_cat_different_id(self):
         """'cat' works with old and new files"""
@@ -138,10 +136,9 @@
         wt.add('README')
         wt.commit('Making sure there is a basis_tree available')
         wt = self.make_branch_and_tree('b')
-        os.chdir('b')
 
-        out, err = self.run_bzr(
-            ['cat', '-r', 'branch:../a', 'README'])
+        out, err = self.run_bzr(['cat', '-r', 'branch:../a', 'README'],
+                                working_dir='b')
         self.assertEqual('contents of a/README\n', out)
 
     def test_cat_filters(self):

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2012-01-05 15:54:17 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2012-01-05 17:56:30 +0000
@@ -133,12 +133,11 @@
     def test_checkout_in_branch_with_r(self):
         branch = _mod_branch.Branch.open('branch')
         branch.bzrdir.destroy_workingtree()
-        os.chdir('branch')
-        self.run_bzr('checkout -r 1')
-        tree = workingtree.WorkingTree.open('.')
+        self.run_bzr('checkout -r 1', working_dir='branch')
+        tree = workingtree.WorkingTree.open('branch')
         self.assertEqual('1', tree.last_revision())
         branch.bzrdir.destroy_workingtree()
-        self.run_bzr('checkout -r 0')
+        self.run_bzr('checkout -r 0', working_dir='branch')
         self.assertEqual('null:', tree.last_revision())
 
     def test_checkout_files_from(self):

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2012-01-05 15:54:17 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2012-01-05 17:56:30 +0000
@@ -328,11 +328,10 @@
         finally:
             other_tree.unlock()
         this_tree.merge_from_branch(other_tree.branch)
-        os.chdir('this')
-        out,err = self.run_bzr('commit -m added')
+        out, err = self.run_bzr('commit -m added', working_dir='this')
         self.assertEqual('', out)
         self.assertEqual(set([
-            'Committing to: %s/' % osutils.getcwd(),
+            'Committing to: %s/' % osutils.pathjoin(osutils.getcwd(), 'this'),
             'modified filetomodify',
             'added newdir',
             'added newfile',
@@ -472,10 +471,10 @@
         t.add(['file-a', 'dir-a', 'dir-a/file-b'])
         t.commit('Create')
         t.remove(['file-a', 'dir-a/file-b'])
-        os.chdir('dir-a')
-        result = self.run_bzr('commit . -m removed-file-b')[1]
+        result = self.run_bzr('commit . -m removed-file-b',
+                              working_dir='dir-a')[1]
         self.assertNotContainsRe(result, 'file-a')
-        result = self.run_bzr('status')[0]
+        result = self.run_bzr('status', working_dir='dir-a')[0]
         self.assertContainsRe(result, 'removed:\n  file-a')
 
     def test_strict_commit(self):
@@ -485,8 +484,7 @@
         self.build_tree(['tree/a'])
         tree.add('a')
         # A simple change should just work
-        self.run_bzr('commit --strict -m adding-a',
-                     working_dir='tree')
+        self.run_bzr('commit --strict -m adding-a', working_dir='tree')
 
     def test_strict_commit_no_changes(self):
         """commit --strict gives "no changes" if there is nothing to commit"""
@@ -830,30 +828,30 @@
         """Ensure commit error if username is not set.
         """
         self.run_bzr(['init', 'foo'])
-        os.chdir('foo')
-        open('foo.txt', 'w').write('hello')
-        self.run_bzr(['add'])
+        with open('foo/foo.txt', 'w') as f:
+            f.write('hello')
+        self.run_bzr(['add'], working_dir='foo')
         self.overrideEnv('EMAIL', None)
         self.overrideEnv('BZR_EMAIL', None)
         # Also, make sure that it's not inferred from mailname.
         self.overrideAttr(config, '_auto_user_id',
             lambda: (None, None))
-        out, err = self.run_bzr(['commit', '-m', 'initial'], 3)
-        self.assertContainsRe(err, 'Unable to determine your name')
+        self.run_bzr_error(
+            ['Unable to determine your name'],
+            ['commit', '-m', 'initial'], working_dir='foo')
 
     def test_commit_recursive_checkout(self):
         """Ensure that a commit to a recursive checkout fails cleanly.
         """
         self.run_bzr(['init', 'test_branch'])
         self.run_bzr(['checkout', 'test_branch', 'test_checkout'])
-        os.chdir('test_checkout')
-        self.run_bzr(['bind', '.']) # bind to self
-        open('foo.txt', 'w').write('hello')
-        self.run_bzr(['add'])
-        out, err = self.run_bzr(['commit', '-m', 'addedfoo'], 3)
-        self.assertEqual(out, '')
-        self.assertContainsRe(err,
-            'Branch.*test_checkout.*appears to be bound to itself')
+        self.run_bzr(['bind', '.'], working_dir='test_checkout') # bind to self
+        with open('test_checkout/foo.txt', 'w') as f:
+            f.write('hello')
+        self.run_bzr(['add'], working_dir='test_checkout')
+        out, err = self.run_bzr_error(
+            ['Branch.*test_checkout.*appears to be bound to itself'],
+            ['commit', '-m', 'addedfoo'], working_dir='test_checkout')
 
     def test_mv_dirs_non_ascii(self):
         """Move directory with non-ascii name and containing files.

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2011-01-19 18:44:11 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2011 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,8 +15,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-"""Black-box tests for bzr diff.
-"""
+"""Black-box tests for bzr diff."""
 
 import os
 import re
@@ -288,8 +287,8 @@
         self.example_branch2()
         self.build_tree_contents([('branch1/file1', 'new line')])
         os.mkdir('branch1/dir1')
-        os.chdir('branch1/dir1')
-        output = self.run_bzr('diff -r 1..', retcode=1)
+        output = self.run_bzr('diff -r 1..', retcode=1,
+                              working_dir='branch1/dir1')
         self.assertContainsRe(output[0], '\n\\-original line\n\\+new line\n')
 
     def test_diff_across_rename(self):
@@ -316,8 +315,7 @@
                 return super(BooDiffTree, self).show_diff(specific_files,
                     extra_trees)
 
-        diff_format_registry.register("boo", BooDiffTree, 
-            "Scary diff format")
+        diff_format_registry.register("boo", BooDiffTree, "Scary diff format")
         self.addCleanup(diff_format_registry.remove, "boo")
         self.make_example_branch()
         self.build_tree_contents([('hello', 'hello world!\n')])
@@ -343,7 +341,8 @@
         return tree
 
     def example_branches(self):
-        branch1_tree, branch2_tree = super(TestCheckoutDiff, self).example_branches()
+        branch1_tree, branch2_tree = super(TestCheckoutDiff,
+                                           self).example_branches()
         os.mkdir('checkouts')
         branch1_tree = branch1_tree.branch.create_checkout('checkouts/branch1')
         branch2_tree = branch2_tree.branch.create_checkout('checkouts/branch2')

=== modified file 'bzrlib/tests/blackbox/test_ls.py'
--- bzrlib/tests/blackbox/test_ls.py	2011-12-14 21:57:37 +0000
+++ bzrlib/tests/blackbox/test_ls.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,14 +16,14 @@
 
 """External tests of 'bzr ls'"""
 
-import os
-
-from bzrlib import ignores
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib import (
+    ignores,
+    tests,
+    )
 from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
-class TestLS(TestCaseWithTransport):
+class TestLS(tests.TestCaseWithTransport):
 
     def setUp(self):
         super(TestLS, self).setUp()
@@ -37,13 +37,13 @@
                                  ('a', 'hello\n'),
                                  ])
 
-    def ls_equals(self, value, args=None, recursive=True):
+    def ls_equals(self, value, args=None, recursive=True, working_dir=None):
         command = 'ls'
         if args is not None:
             command += ' ' + args
         if recursive:
             command += ' -R'
-        out, err = self.run_bzr(command)
+        out, err = self.run_bzr(command, working_dir=working_dir)
         self.assertEqual('', err)
         self.assertEqualDiff(value, out)
 
@@ -127,16 +127,13 @@
                        , '--verbose', recursive=False)
 
         # Check what happens in a sub-directory
-        os.chdir('subdir')
-        self.ls_equals('b\n')
-        self.ls_equals('b\0'
-                  , '--null')
-        self.ls_equals('subdir/b\n'
-                       , '--from-root')
-        self.ls_equals('subdir/b\0'
-                       , '--from-root --null')
-        self.ls_equals('subdir/b\n'
-                       , '--from-root', recursive=False)
+        self.ls_equals('b\n', working_dir='subdir')
+        self.ls_equals('b\0', '--null', working_dir='subdir')
+        self.ls_equals('subdir/b\n', '--from-root', working_dir='subdir')
+        self.ls_equals('subdir/b\0', '--from-root --null',
+                       working_dir='subdir')
+        self.ls_equals('subdir/b\n', '--from-root', recursive=False,
+                       working_dir='subdir')
 
     def test_ls_path(self):
         """If a path is specified, files are listed with that prefix"""
@@ -144,24 +141,23 @@
         self.wt.add(['subdir', 'subdir/b'])
         self.ls_equals('subdir/b\n' ,
                        'subdir')
-        os.chdir('subdir')
         self.ls_equals('../.bzrignore\n'
                        '../a\n'
                        '../subdir/\n'
                        '../subdir/b\n' ,
-                       '..')
+                       '..', working_dir='subdir')
         self.ls_equals('../.bzrignore\0'
                        '../a\0'
                        '../subdir\0'
                        '../subdir/b\0' ,
-                       '.. --null')
+                       '.. --null', working_dir='subdir')
         self.ls_equals('?        ../.bzrignore\n'
                        '?        ../a\n'
                        'V        ../subdir/\n'
                        'V        ../subdir/b\n' ,
-                       '.. --verbose')
+                       '.. --verbose', working_dir='subdir')
         self.run_bzr_error(['cannot specify both --from-root and PATH'],
-                           'ls --from-root ..')
+                           'ls --from-root ..', working_dir='subdir')
 
     def test_ls_revision(self):
         self.wt.add(['a'])
@@ -174,8 +170,7 @@
         self.ls_equals('V        a\n'
                        , '--verbose --revision 1')
 
-        os.chdir('subdir')
-        self.ls_equals('', '--revision 1')
+        self.ls_equals('', '--revision 1', working_dir='subdir')
 
     def test_ls_branch(self):
         """If a branch is specified, files are listed from it"""
@@ -247,7 +242,7 @@
         self.ls_equals('sub/file\n', '-d dir sub')
 
 
-class TestSmartServerLs(TestCaseWithTransport):
+class TestSmartServerLs(tests.TestCaseWithTransport):
 
     def test_simple_ls(self):
         self.setup_smart_server_with_call_log()

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2012-01-05 15:54:17 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2012-01-05 17:56:30 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2011 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -88,45 +88,49 @@
 
         self.build_tree_contents([('a/hello', 'quuux')])
         # We can't merge when there are in-tree changes
-        os.chdir('a')
-        self.run_bzr('merge ../b', retcode=3)
-        a = workingtree.WorkingTree.open('.')
+        self.run_bzr('merge ../b', retcode=3, working_dir='a')
+        a = workingtree.WorkingTree.open('a')
         a_tip = a.commit("Like an epidemic of u's")
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type blooof',
-                    retcode=3)
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type merge3')
-        a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type weave')
-        a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1..last:1 --merge-type lca')
-        a_tree.revert(backups=False)
+
+        def run_merge_then_revert(args, retcode=None, working_dir='a'):
+            self.run_bzr(['merge', '../b', '-r', 'last:1..last:1'] + args,
+                         retcode=retcode, working_dir=working_dir)
+            if retcode != 3:
+                a_tree.revert(backups=False)
+
+        run_merge_then_revert(['--merge-type', 'bloof'], retcode=3)
+        run_merge_then_revert(['--merge-type', 'merge3'])
+        run_merge_then_revert(['--merge-type', 'weave'])
+        run_merge_then_revert(['--merge-type', 'lca'])
         self.run_bzr_error(['Show-base is not supported for this merge type'],
                            'merge ../b -r last:1..last:1 --merge-type weave'
-                           ' --show-base')
-        a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1..last:1 --reprocess')
-        a_tree.revert(backups=False)
-        self.run_bzr('merge ../b -r last:1')
-        self.check_file_contents('goodbye', 'quux')
+                           ' --show-base', working_dir='a')
+        a_tree.revert(backups=False)
+        self.run_bzr('merge ../b -r last:1..last:1 --reprocess',
+                     working_dir='a')
+        a_tree.revert(backups=False)
+        self.run_bzr('merge ../b -r last:1', working_dir='a')
+        self.check_file_contents('a/goodbye', 'quux')
         # Merging a branch pulls its revision into the tree
-        b = branch.Branch.open('../b')
+        b = branch.Branch.open('b')
         b_tip = b.last_revision()
         self.assertTrue(a.branch.repository.has_revision(b_tip))
         self.assertEqual([a_tip, b_tip], a.get_parent_ids())
         a_tree.revert(backups=False)
-        out, err = self.run_bzr('merge -r revno:1:./hello', retcode=3)
+        out, err = self.run_bzr('merge -r revno:1:./hello', retcode=3,
+                                working_dir='a')
         self.assertTrue("Not a branch" in err)
         self.run_bzr('merge -r revno:%d:./..revno:%d:../b'
-                    %(ancestor,b.revno()))
+                    %(ancestor,b.revno()), working_dir='a')
         self.assertEquals(a.get_parent_ids(),
                           [a.branch.last_revision(), b.last_revision()])
-        self.check_file_contents('goodbye', 'quux')
+        self.check_file_contents('a/goodbye', 'quux')
         a_tree.revert(backups=False)
-        self.run_bzr('merge -r revno:%d:../b'%b.revno())
+        self.run_bzr('merge -r revno:%d:../b'%b.revno(), working_dir='a')
         self.assertEquals(a.get_parent_ids(),
                           [a.branch.last_revision(), b.last_revision()])
         a_tip = a.commit('merged')
-        self.run_bzr('merge ../b -r last:1')
+        self.run_bzr('merge ../b -r last:1', working_dir='a')
         self.assertEqual([a_tip], a.get_parent_ids())
 
     def test_merge_defaults_to_reprocess(self):
@@ -211,14 +215,14 @@
             ('b/b.txt', 'hello\nsomething\n'),
             ('b/sub/c.txt', 'hello\nsomething\n')])
         b_tree.commit(message='Modified a.txt')
-        os.chdir('b')
-        self.run_bzr('merge ../a/', retcode=1)
-        self.assertPathExists('sub/a.txt.THIS')
-        self.assertPathExists('sub/a.txt.BASE')
-        os.chdir('../a')
-        self.run_bzr('merge ../b/', retcode=1)
-        self.assertPathExists('sub/a.txt.OTHER')
-        self.assertPathExists('sub/a.txt.BASE')
+
+        self.run_bzr('merge ../a/', retcode=1, working_dir='b')
+        self.assertPathExists('b/sub/a.txt.THIS')
+        self.assertPathExists('b/sub/a.txt.BASE')
+
+        self.run_bzr('merge ../b/', retcode=1, working_dir='a')
+        self.assertPathExists('a/sub/a.txt.OTHER')
+        self.assertPathExists('a/sub/a.txt.BASE')
 
     def test_conflict_leaves_base_this_other_files(self):
         tree, other = self.create_conflicting_branches()
@@ -258,20 +262,19 @@
         branch_b.set_parent(None)
         self.assertEqual(None, branch_b.get_parent())
         # test merge for failure without parent set
-        os.chdir('branch_b')
-        out = self.run_bzr('merge', retcode=3)
+        out = self.run_bzr('merge', retcode=3, working_dir='branch_b')
         self.assertEquals(out,
                 ('','bzr: ERROR: No location specified or remembered\n'))
 
         # test uncommitted changes
-        self.build_tree(['d'])
+        self.build_tree(['branch_b/d'])
         tree_b.add('d')
         self.run_bzr_error(['Working tree ".*" has uncommitted changes'],
-                           'merge')
+                           'merge', working_dir='branch_b')
 
         # merge should now pass and implicitly remember merge location
         tree_b.commit('commit d')
-        out, err = self.run_bzr('merge ../branch_a')
+        out, err = self.run_bzr('merge ../branch_a', working_dir='branch_b')
 
         base = urlutils.local_path_from_url(branch_a.base)
         self.assertEndsWith(err, '+N  b\nAll changes applied successfully.\n')
@@ -280,17 +283,18 @@
         self.assertEquals(osutils.abspath(branch_b.get_submit_branch()),
                           osutils.abspath(parent))
         # test implicit --remember when committing new file
-        self.build_tree(['e'])
+        self.build_tree(['branch_b/e'])
         tree_b.add('e')
         tree_b.commit('commit e')
-        out, err = self.run_bzr('merge')
+        out, err = self.run_bzr('merge', working_dir='branch_b')
         self.assertStartsWith(
             err, 'Merging from remembered submit location %s\n' % (base,))
         # re-open tree as external run_bzr modified it
         tree_b = branch_b.bzrdir.open_workingtree()
         tree_b.commit('merge branch_a')
         # test explicit --remember
-        out, err = self.run_bzr('merge ../branch_c --remember')
+        out, err = self.run_bzr('merge ../branch_c --remember',
+                                working_dir='branch_b')
         self.assertEquals(out, '')
         self.assertEquals(err, '+N  c\nAll changes applied successfully.\n')
         # re-open branch as external run_bzr modified it
@@ -314,10 +318,8 @@
 
         self.build_tree_contents([('branch_b/a', 'goodbye')])
         tree_b.commit('message')
-        os.chdir('branch_b')
-        self.run_bzr('bundle ../branch_a -o ../bundle')
-        os.chdir('../branch_a')
-        self.run_bzr('merge ../bundle', retcode=1)
+        self.run_bzr('bundle ../branch_a -o ../bundle', working_dir='branch_b')
+        self.run_bzr('merge ../bundle', retcode=1, working_dir='branch_a')
         testament_a = Testament.from_revision(tree_a.branch.repository,
                                               tree_b.get_parent_ids()[0])
         testament_b = Testament.from_revision(tree_b.branch.repository,
@@ -327,10 +329,10 @@
         tree_a.set_conflicts(conflicts.ConflictList())
         tree_a.commit('message')
         # it is legal to attempt to merge an already-merged bundle
-        output = self.run_bzr('merge ../bundle')[1]
+        err = self.run_bzr('merge ../bundle', working_dir='branch_a')[1]
         # but it does nothing
         self.assertFalse(tree_a.changes_from(tree_a.basis_tree()).has_changed())
-        self.assertEqual('Nothing to do.\n', output)
+        self.assertEqual('Nothing to do.\n', err)
 
     def test_merge_uncommitted(self):
         """Check that merge --uncommitted behaves properly"""
@@ -343,7 +345,6 @@
         tree_a.rename_one('file_1', 'file_i')
         tree_a.commit('commit 2')
         tree_a.rename_one('file_2', 'file_ii')
-        ## os.chdir('b')
         self.run_bzr('merge a --uncommitted -d b')
         self.assertPathExists('b/file_1')
         self.assertPathExists('b/file_ii')
@@ -358,10 +359,10 @@
         tree_a.bzrdir.sprout('tree_b')
         self.build_tree(['tree_a/file1', 'tree_a/file2'])
         tree_a.add(['file1', 'file2'])
-        os.chdir('tree_b')
-        self.run_bzr(['merge', '--uncommitted', '../tree_a/file1'])
-        self.assertPathExists('file1')
-        self.assertPathDoesNotExist('file2')
+        self.run_bzr(['merge', '--uncommitted', '../tree_a/file1'],
+                     working_dir='tree_b')
+        self.assertPathExists('tree_b/file1')
+        self.assertPathDoesNotExist('tree_b/file2')
 
     def test_merge_nonexistent_file(self):
         """It should not be possible to merge changes from a file which
@@ -370,9 +371,8 @@
         self.build_tree_contents([('tree_a/file', 'bar\n')])
         tree_a.add(['file'])
         tree_a.commit('commit 1')
-        os.chdir('tree_a')
         self.run_bzr_error(('Path\(s\) do not exist: non/existing',),
-                           ['merge', 'non/existing'])
+                           ['merge', 'non/existing'], working_dir='tree_a')
 
     def pullable_branch(self):
         tree_a = self.make_branch_and_tree('a')
@@ -387,10 +387,9 @@
 
     def test_merge_pull(self):
         self.pullable_branch()
-        os.chdir('a')
-        (out, err) = self.run_bzr('merge --pull ../b')
+        (out, err) = self.run_bzr('merge --pull ../b', working_dir='a')
         self.assertContainsRe(out, 'Now on revision 2\\.')
-        tree_a = workingtree.WorkingTree.open('.')
+        tree_a = workingtree.WorkingTree.open('a')
         self.assertEqual([self.id2], tree_a.get_parent_ids())
 
     def test_merge_pull_preview(self):
@@ -417,14 +416,13 @@
         os.unlink('tree_a/file')
         self.build_tree(['tree_a/file/'])
         tree_a.commit('changed file to directory')
-        os.chdir('tree_b')
-        self.run_bzr('merge ../tree_a')
-        self.assertEqual('directory', osutils.file_kind('file'))
+        self.run_bzr('merge ../tree_a', working_dir='tree_b')
+        self.assertEqual('directory', osutils.file_kind('tree_b/file'))
         tree_b.revert()
-        self.assertEqual('file', osutils.file_kind('file'))
-        self.build_tree_contents([('file', 'content_2')])
+        self.assertEqual('file', osutils.file_kind('tree_b/file'))
+        self.build_tree_contents([('tree_b/file', 'content_2')])
         tree_b.commit('content change')
-        self.run_bzr('merge ../tree_a', retcode=1)
+        self.run_bzr('merge ../tree_a', retcode=1, working_dir='tree_b')
         self.assertEqual(tree_b.conflicts(),
                          [conflicts.ContentsConflict('file',
                                                      file_id='file-id')])

=== modified file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	2011-09-16 15:37:58 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,15 +16,13 @@
 
 """Black-box tests for bzr missing."""
 
-import os
-
-from bzrlib import osutils
-
-from bzrlib.branch import Branch
-from bzrlib.tests import TestCaseWithTransport
-
-
-class TestMissing(TestCaseWithTransport):
+from bzrlib import (
+    osutils,
+    tests,
+    )
+
+
+class TestMissing(tests.TestCaseWithTransport):
 
     def assertMessages(self, out, must_have=(), must_not_have=()):
         """Check if commit messages are in or not in the output"""
@@ -49,7 +47,8 @@
         self.assertEqual('', err)
 
     def test_missing(self):
-        missing = "You are missing 1 revision:"
+        missing_one = "You are missing 1 revision:"
+        extra_one = "You have 1 extra revision:"
 
         # create a source branch
         a_tree = self.make_branch_and_tree('a')
@@ -62,6 +61,21 @@
         self.build_tree_contents([('b/a', 'initial\nmore\n')])
         b_tree.commit(message='more')
 
+        def run_missing(args, retcode=1, working_dir=None):
+            out, err = self.run_bzr(['missing'] + args,
+                                    retcode=retcode, working_dir=working_dir)
+            # we do not expect any error output.
+            self.assertEqual('', err)
+            return out.splitlines()
+
+        def run_missing_a(args, retcode=1):
+            return run_missing(['../a'] + args,
+                               retcode=retcode, working_dir='b')
+
+        def run_missing_b(args, retcode=1):
+            return run_missing(['../b'] + args,
+                               retcode=retcode, working_dir='a')
+
         # run missing in a against b
         # this should not require missing to take out a write lock on a
         # or b. So we take a write lock on both to test that at the same
@@ -72,16 +86,13 @@
         a_branch.lock_write()
         b_branch = b_tree.branch
         b_branch.lock_write()
-        os.chdir('a')
-        out,err = self.run_bzr('missing ../b', retcode=1)
-        lines = out.splitlines()
+
+        lines = run_missing_b([])
         # we're missing the extra revision here
-        self.assertEqual(missing, lines[0])
+        self.assertEqual(missing_one, lines[0])
         # and we expect 8 lines of output which we trust at the moment to be
         # good.
         self.assertEqual(8, len(lines))
-        # we do not expect any error output.
-        self.assertEqual('', err)
         # unlock the branches for the rest of the test
         a_branch.unlock()
         b_branch.unlock()
@@ -91,61 +102,60 @@
         a_tree.commit(message='merge')
 
         # compare again, but now we have the 'merge' commit extra
-        lines = self.run_bzr('missing ../b', retcode=1)[0].splitlines()
-        self.assertEqual("You have 1 extra revision:", lines[0])
-        self.assertEqual(8, len(lines))
-        lines2 = self.run_bzr('missing ../b --mine-only', retcode=1)[0]
-        lines2 = lines2.splitlines()
+        lines = run_missing_b([])
+        self.assertEqual(extra_one, lines[0])
+        self.assertLength(8, lines)
+
+        lines2 = run_missing_b(['--mine-only'])
         self.assertEqual(lines, lines2)
-        lines3 = self.run_bzr('missing ../b --theirs-only', retcode=0)[0]
-        self.assertEqualDiff('Other branch has no new revisions.\n', lines3)
+
+        lines3 = run_missing_b(['--theirs-only'], retcode=0)
+        self.assertEqualDiff('Other branch has no new revisions.', lines3[0])
 
         # relative to a, missing the 'merge' commit
-        os.chdir('../b')
-        lines = self.run_bzr('missing ../a', retcode=1)[0].splitlines()
-        self.assertEqual(missing, lines[0])
-        self.assertEqual(8, len(lines))
-        lines2 = self.run_bzr('missing ../a --theirs-only', retcode=1)[0]
-        lines2 = lines2.splitlines()
+        lines = run_missing_a([])
+        self.assertEqual(missing_one, lines[0])
+        self.assertLength(8, lines)
+
+        lines2 = run_missing_a(['--theirs-only'])
         self.assertEqual(lines, lines2)
-        lines3 = self.run_bzr('missing ../a --mine-only', retcode=0)[0]
-        self.assertEqualDiff('This branch has no new revisions.\n', lines3)
-        lines4 = self.run_bzr('missing ../a --short', retcode=1)[0]
-        lines4 = lines4.splitlines()
-        self.assertEqual(4, len(lines4))
-        lines4a = self.run_bzr('missing ../a -S', retcode=1)[0]
-        lines4a = lines4a.splitlines()
+
+        lines3 = run_missing_a(['--mine-only'], retcode=0)
+        self.assertEqualDiff('This branch has no new revisions.', lines3[0])
+
+        lines4 = run_missing_a(['--short'])
+        self.assertLength(4, lines4)
+
+        lines4a = run_missing_a(['-S'])
         self.assertEqual(lines4, lines4a)
-        lines5 = self.run_bzr('missing ../a --line', retcode=1)[0]
-        lines5 = lines5.splitlines()
-        self.assertEqual(2, len(lines5))
-        lines6 = self.run_bzr('missing ../a --reverse', retcode=1)[0]
-        lines6 = lines6.splitlines()
+
+        lines5 = run_missing_a(['--line'])
+        self.assertLength(2, lines5)
+
+        lines6 = run_missing_a(['--reverse'])
         self.assertEqual(lines6, lines)
-        lines7 = self.run_bzr('missing ../a --show-ids', retcode=1)[0]
-        lines7 = lines7.splitlines()
-        self.assertEqual(11, len(lines7))
-        lines8 = self.run_bzr('missing ../a --verbose', retcode=1)[0]
-        lines8 = lines8.splitlines()
+
+        lines7 = run_missing_a(['--show-ids'])
+        self.assertLength(11, lines7)
+
+        lines8 =  run_missing_a(['--verbose'])
         self.assertEqual("modified:", lines8[-2])
         self.assertEqual("  a", lines8[-1])
 
-        os.chdir('../a')
-        self.assertEqualDiff('Other branch has no new revisions.\n',
-                             self.run_bzr('missing ../b --theirs-only')[0])
+        self.assertEqualDiff('Other branch has no new revisions.',
+                             run_missing_b(['--theirs-only'], retcode=0)[0])
 
         # after a pull we're back on track
         b_tree.pull(a_branch)
-        self.assertEqualDiff("Branches are up to date.\n",
-                             self.run_bzr('missing ../b')[0])
-        os.chdir('../b')
-        self.assertEqualDiff('Branches are up to date.\n',
-                             self.run_bzr('missing ../a')[0])
+        self.assertEqualDiff("Branches are up to date.",
+                             run_missing_b([], retcode=0)[0])
+        self.assertEqualDiff('Branches are up to date.',
+                             run_missing_a([], retcode=0)[0])
         # If you supply mine or theirs you only know one side is up to date
-        self.assertEqualDiff('This branch has no new revisions.\n',
-                             self.run_bzr('missing ../a --mine-only')[0])
-        self.assertEqualDiff('Other branch has no new revisions.\n',
-                             self.run_bzr('missing ../a --theirs-only')[0])
+        self.assertEqualDiff('This branch has no new revisions.',
+                             run_missing_a(['--mine-only'], retcode=0)[0])
+        self.assertEqualDiff('Other branch has no new revisions.',
+                              run_missing_a(['--theirs-only'], retcode=0)[0])
 
     def test_missing_filtered(self):
         # create a source branch
@@ -160,24 +170,27 @@
             a_tree.commit(message='a%d' % i)
             b_tree.commit(message='b%d' % i)
 
-        os.chdir('a')
         # local
-        out,err = self.run_bzr('missing ../b --my-revision 3', retcode=1)
+        out,err = self.run_bzr('missing ../b --my-revision 3',
+                               retcode=1, working_dir='a')
         self.assertMessages(out, ('a3', 'b2', 'b3', 'b4', 'b5'), ('a2', 'a4'))
 
-        out,err = self.run_bzr('missing ../b --my-revision 3..4', retcode=1)
+        out,err = self.run_bzr('missing ../b --my-revision 3..4',
+                               retcode=1, working_dir='a')
         self.assertMessages(out, ('a3', 'a4'), ('a2', 'a5'))
 
         #remote
-        out,err = self.run_bzr('missing ../b -r 3', retcode=1)
+        out,err = self.run_bzr('missing ../b -r 3',
+                               retcode=1, working_dir='a')
         self.assertMessages(out, ('a2', 'a3', 'a4', 'a5', 'b3'), ('b2', 'b4'))
 
-        out,err = self.run_bzr('missing ../b -r 3..4', retcode=1)
+        out,err = self.run_bzr('missing ../b -r 3..4',
+                               retcode=1, working_dir='a')
         self.assertMessages(out, ('b3', 'b4'), ('b2', 'b5'))
 
         #both
         out,err = self.run_bzr('missing ../b --my-revision 3..4 -r 3..4',
-            retcode=1)
+                               retcode=1, working_dir='a')
         self.assertMessages(out, ('a3', 'a4', 'b3', 'b4'),
             ('a2', 'a5', 'b2', 'b5'))
 
@@ -191,14 +204,13 @@
         wt.add('foo')
         wt.commit('initial')
 
-        os.chdir('a')
-        location = osutils.getcwd() + '/'
+        location = osutils.getcwd() + '/a/'
 
         # clone
-        b.bzrdir.sprout('../b')
+        b.bzrdir.sprout('b')
 
         # check last location
-        lines, err = self.run_bzr('missing', working_dir='../b')
+        lines, err = self.run_bzr('missing', working_dir='b')
         self.assertEquals('Using saved parent location: %s\n'
                           'Branches are up to date.\n' % location,
                           lines)
@@ -219,7 +231,6 @@
         b_tree.commit(message='more')
 
         out2, err2 = self.run_bzr('missing --directory a b', retcode=1)
-        os.chdir('a')
-        out1, err1 = self.run_bzr('missing ../b', retcode=1)
+        out1, err1 = self.run_bzr('missing ../b', retcode=1, working_dir='a')
         self.assertEqualDiff(out1, out2)
         self.assertEqualDiff(err1, err2)

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2011-12-06 12:36:26 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -120,13 +120,10 @@
         tree = self.make_branch_and_tree('.')
         tree.add(['sub1', 'sub1/sub2', 'sub1/hello.txt'])
 
-        os.chdir('sub1/sub2')
-        self.run_bzr('mv ../hello.txt .')
-        self.assertPathExists('./hello.txt')
+        self.run_bzr('mv ../hello.txt .', working_dir='sub1/sub2')
+        self.assertPathExists('sub1/sub2/hello.txt')
 
-        os.chdir('..')
-        self.run_bzr('mv sub2/hello.txt .')
-        os.chdir('..')
+        self.run_bzr('mv sub2/hello.txt .', working_dir='sub1')
         self.assertMoved('sub1/sub2/hello.txt','sub1/hello.txt')
 
     def test_mv_change_case_file(self):

=== modified file 'bzrlib/tests/blackbox/test_nick.py'
--- bzrlib/tests/blackbox/test_nick.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_nick.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2010, 2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,74 +16,75 @@
 
 """Black-box tests for bzr nick."""
 
-import os
-
 import bzrlib
-from bzrlib import osutils
-from bzrlib.tests import TestCaseWithTransport
-
-
-class TestNick(TestCaseWithTransport):
+from bzrlib import (
+    branch,
+    osutils,
+    tests,
+    )
+
+
+class TestNick(tests.TestCaseWithTransport):
+
+    def assertNick(self, expected, working_dir='.', explicit=None,
+                   directory=None):
+        cmd = ['nick']
+        if directory is not None:
+            cmd.extend(['--directory', directory])
+        # The nick command outputs the nick on a single line
+        actual = self.run_bzr(cmd, working_dir=working_dir)[0][:-1]
+        self.assertEquals(expected, actual)
+        if explicit is not None:
+            br = branch.Branch.open(working_dir)
+            conf = br.get_config()
+            self.assertEquals(explicit, conf.has_explicit_nickname())
+            if explicit:
+                self.assertEquals(expected, conf._get_explicit_nickname())
 
     def test_nick_command(self):
         """bzr nick for viewing, setting nicknames"""
         self.make_branch_and_tree('me.dev')
-        os.chdir('me.dev')
-        nick = self.run_bzr('nick')[0]
-        self.assertEqual('me.dev\n', nick)
+        self.assertNick('me.dev', working_dir='me.dev')
         # set the nickname
-        self.run_bzr("nick moo")
-        nick = self.run_bzr('nick')[0]
-        self.assertEqual('moo\n', nick)
+        self.run_bzr("nick moo", working_dir='me.dev')
+        self.assertNick('moo', working_dir='me.dev')
 
     def test_autonick_urlencoded(self):
         # https://bugs.launchpad.net/bzr/+bug/66857 -- nick was printed
         # urlencoded but shouldn't be
         self.make_branch_and_tree('!repo')
-        os.chdir('!repo')
-        nick = self.run_bzr('nick')[0]
-        self.assertEqual('!repo\n', nick)
+        self.assertNick('!repo', working_dir='!repo')
 
     def test_bound_nick(self):
-        """Check that nick works well for checkouts."""
-        base = self.make_branch_and_tree('base')
-        child = self.make_branch_and_tree('child')
-        os.chdir('child')
-        self.assertEqual('child', self.run_bzr('nick')[0][:-1])
-        self.assertEqual(False,
-                         child.branch.get_config().has_explicit_nickname())
-        self.run_bzr('bind ../base')
-        self.assertEqual(base.branch.nick, self.run_bzr('nick')[0][:-1])
-        self.assertEqual(False,
-                         child.branch.get_config().has_explicit_nickname())
-
-        self.run_bzr('unbind')
-        self.run_bzr("nick explicit_nick")
-        self.assertEqual("explicit_nick", self.run_bzr('nick')[0][:-1])
-        self.assertEqual("explicit_nick",
-                         child.branch.get_config()._get_explicit_nickname())
-        self.run_bzr('bind ../base')
-        self.assertEqual(base.branch.nick, self.run_bzr('nick')[0][:-1])
-        self.assertEqual(base.branch.nick,
-                         child.branch.get_config()._get_explicit_nickname())
+        """Bind should not update implicit nick."""
+        base = self.make_branch_and_tree('base')
+        child = self.make_branch_and_tree('child')
+        self.assertNick('child', working_dir='child', explicit=False)
+
+        self.run_bzr('bind ../base', working_dir='child')
+        self.assertNick(base.branch.nick, working_dir='child', explicit=False)
+
+    def test_bound_nick_explicit(self):
+        """Bind should update explicit nick."""
+        base = self.make_branch_and_tree('base')
+        child = self.make_branch_and_tree('child')
+        self.run_bzr("nick explicit_nick", working_dir='child')
+        self.assertNick('explicit_nick', working_dir='child', explicit=True)
+        self.run_bzr('bind ../base', working_dir='child')
+        self.assertNick(base.branch.nick, working_dir='child', explicit=True)
 
     def test_boundless_nick(self):
         """Nick defaults to implicit local nick when bound branch is AWOL"""
         base = self.make_branch_and_tree('base')
         child = self.make_branch_and_tree('child')
-        os.chdir('child')
-        self.run_bzr('bind ../base')
-        self.assertEqual(base.branch.nick, self.run_bzr('nick')[0][:-1])
-        self.assertEqual(False,
-                         child.branch.get_config().has_explicit_nickname())
-        osutils.rmtree('../base')
-        self.assertEqual('child', self.run_bzr('nick')[0][:-1])
+        self.run_bzr('bind ../base', working_dir='child')
+        self.assertNick(base.branch.nick, working_dir='child', explicit=False)
+        osutils.rmtree('base')
+        self.assertNick('child', working_dir='child', explicit=False)
 
     def test_nick_directory(self):
         """Test --directory option"""
         self.make_branch_and_tree('me.dev')
-        nick = self.run_bzr(['nick', '--directory=me.dev'])[0]
-        self.assertEqual('me.dev\n', nick)
+        self.assertNick('me.dev', directory='me.dev')
         self.run_bzr(['nick', '-d', 'me.dev', 'moo'])
-        nick = self.run_bzr(['nick', '--directory', 'me.dev'])[0]
-        self.assertEqual('moo\n', nick)
+        self.assertNick('moo', directory='me.dev')

=== modified file 'bzrlib/tests/blackbox/test_pack.py'
--- bzrlib/tests/blackbox/test_pack.py	2011-12-14 21:57:37 +0000
+++ bzrlib/tests/blackbox/test_pack.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2007, 2009-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,8 @@
         for i in range(total_lines):
             text += line_prefix + str(i+1) + "\n"
 
-        open(path, 'w').write(text)
+        with open(path, 'w') as f:
+            f.write(text)
         if versioned:
             self.run_bzr(['add', path])
             self.run_bzr(['ci', '-m', '"' + path + '"'])
@@ -69,10 +70,8 @@
     def test_pack_clean_obsolete_packs(self):
         """Ensure --clean-obsolete-packs removes obsolete pack files
         """
-        wd = 'foobar0'
-        wt = self.make_branch_and_tree(wd)
-        transport = wt.branch.repository.bzrdir.transport
-        os.chdir(wd)
+        wt = self.make_branch_and_tree('.')
+        t = wt.branch.repository.bzrdir.transport
 
         # do multiple commits to ensure that obsolete packs are created
         # by 'bzr pack'
@@ -82,7 +81,7 @@
 
         out, err = self.run_bzr(['pack', '--clean-obsolete-packs'])
 
-        pack_names = transport.list_dir('repository/obsolete_packs')
+        pack_names = t.list_dir('repository/obsolete_packs')
         self.assertTrue(len(pack_names) == 0)
 
 

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2012-01-05 15:54:17 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2012-01-05 17:56:30 +0000
@@ -23,20 +23,19 @@
 from bzrlib import (
     branch,
     debug,
+    osutils,
     remote,
     tests,
+    uncommit,
     urlutils,
+    workingtree,
     )
 
-from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
-from bzrlib.osutils import pathjoin
 from bzrlib.tests import (
     fixtures,
     script,
     )
-from bzrlib.uncommit import uncommit
-from bzrlib.workingtree import WorkingTree
 
 
 class TestPull(tests.TestCaseWithTransport):
@@ -44,8 +43,8 @@
     def example_branch(self, path='.'):
         tree = self.make_branch_and_tree(path)
         self.build_tree_contents([
-            (pathjoin(path, 'hello'),   'foo'),
-            (pathjoin(path, 'goodbye'), 'baz')])
+            (osutils.pathjoin(path, 'hello'),   'foo'),
+            (osutils.pathjoin(path, 'goodbye'), 'baz')])
         tree.add('hello')
         tree.commit(message='setup')
         tree.add('goodbye')
@@ -56,63 +55,53 @@
         """Pull changes from one branch to another."""
         a_tree = self.example_branch('a')
         base_rev = a_tree.branch.last_revision()
-        os.chdir('a')
-        self.run_bzr('pull', retcode=3)
-        self.run_bzr('missing', retcode=3)
-        self.run_bzr('missing .')
-        self.run_bzr('missing')
+        self.run_bzr('pull', retcode=3, working_dir='a')
+        self.run_bzr('missing', retcode=3, working_dir='a')
+        self.run_bzr('missing .', working_dir='a')
+        self.run_bzr('missing', working_dir='a')
         # this will work on windows because we check for the same branch
         # in pull - if it fails, it is a regression
-        self.run_bzr('pull')
-        self.run_bzr('pull /', retcode=3)
+        self.run_bzr('pull', working_dir='a')
+        self.run_bzr('pull /', retcode=3, working_dir='a')
         if sys.platform not in ('win32', 'cygwin'):
-            self.run_bzr('pull')
+            self.run_bzr('pull', working_dir='a')
 
-        os.chdir('..')
         b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
-        os.chdir('b')
-        self.run_bzr('pull')
-        os.mkdir('subdir')
+        self.run_bzr('pull', working_dir='b')
+        os.mkdir('b/subdir')
         b_tree.add('subdir')
         new_rev = b_tree.commit(message='blah', allow_pointless=True)
 
-        os.chdir('..')
-        a = Branch.open('a')
-        b = Branch.open('b')
+        a = branch.Branch.open('a')
+        b = branch.Branch.open('b')
         self.assertEqual(a.last_revision(), base_rev)
         self.assertEqual(b.last_revision(), new_rev)
 
-        os.chdir('a')
-        self.run_bzr('pull ../b')
+        self.run_bzr('pull ../b', working_dir='a')
         self.assertEqual(a.last_revision(), b.last_revision())
         a_tree.commit(message='blah2', allow_pointless=True)
         b_tree.commit(message='blah3', allow_pointless=True)
         # no overwrite
-        os.chdir('../b')
-        self.run_bzr('pull ../a', retcode=3)
-        os.chdir('..')
+        self.run_bzr('pull ../a', retcode=3, working_dir='b')
         b_tree.bzrdir.sprout('overwriteme')
-        os.chdir('overwriteme')
-        self.run_bzr('pull --overwrite ../a')
-        overwritten = Branch.open('.')
+        self.run_bzr('pull --overwrite ../a', working_dir='overwriteme')
+        overwritten = branch.Branch.open('overwriteme')
         self.assertEqual(overwritten.last_revision(),
                          a.last_revision())
         a_tree.merge_from_branch(b_tree.branch)
         a_tree.commit(message="blah4", allow_pointless=True)
-        os.chdir('../b/subdir')
-        self.run_bzr('pull ../../a')
+
+        self.run_bzr('pull ../../a', working_dir='b/subdir')
         self.assertEqual(a.last_revision(), b.last_revision())
-        sub_tree = WorkingTree.open_containing('.')[0]
+        sub_tree = workingtree.WorkingTree.open_containing('b/subdir')[0]
         sub_tree.commit(message="blah5", allow_pointless=True)
         sub_tree.commit(message="blah6", allow_pointless=True)
-        os.chdir('..')
-        self.run_bzr('pull ../a')
-        os.chdir('../a')
+        self.run_bzr('pull ../a', working_dir='b')
         a_tree.commit(message="blah7", allow_pointless=True)
         a_tree.merge_from_branch(b_tree.branch)
         a_tree.commit(message="blah8", allow_pointless=True)
-        self.run_bzr('pull ../b')
-        self.run_bzr('pull ../b')
+        self.run_bzr('pull ../b', working_dir='a')
+        self.run_bzr('pull ../b', working_dir='a')
 
     def test_pull_dash_d(self):
         self.example_branch('a')
@@ -138,15 +127,14 @@
 
         b_tree = a_tree.bzrdir.sprout('b',
                    revision_id=a_tree.branch.get_rev_id(1)).open_workingtree()
-        os.chdir('b')
-        self.run_bzr('pull -r 2')
-        a = Branch.open('../a')
-        b = Branch.open('.')
+        self.run_bzr('pull -r 2', working_dir='b')
+        a = branch.Branch.open('a')
+        b = branch.Branch.open('b')
         self.assertEqual(a.revno(),4)
         self.assertEqual(b.revno(),2)
-        self.run_bzr('pull -r 3')
+        self.run_bzr('pull -r 3', working_dir='b')
         self.assertEqual(b.revno(),3)
-        self.run_bzr('pull -r 4')
+        self.run_bzr('pull -r 4', working_dir='b')
         self.assertEqual(a.last_revision(), b.last_revision())
 
     def test_pull_tags(self):
@@ -190,8 +178,7 @@
 
         self.assertEqual(b_tree.branch.last_revision_info()[0], 2)
 
-        os.chdir('b')
-        self.run_bzr('pull --overwrite ../a')
+        self.run_bzr('pull --overwrite ../a', working_dir='b')
         (last_revinfo_b) = b_tree.branch.last_revision_info()
         self.assertEqual(last_revinfo_b[0], 3)
         self.assertEqual(last_revinfo_b[1], a_tree.branch.last_revision())
@@ -227,15 +214,14 @@
 
         # With convergence, we could just pull over the
         # new change, but with --overwrite, we want to switch our history
-        os.chdir('b')
-        self.run_bzr('pull --overwrite ../a')
+        self.run_bzr('pull --overwrite ../a', working_dir='b')
         rev_info_b = b_tree.branch.last_revision_info()
         self.assertEqual(rev_info_b[0], 4)
         self.assertEqual(rev_info_b, rev_info_a)
 
     def test_pull_remember(self):
         """Pull changes from one branch to another and test parent location."""
-        transport = self.get_transport()
+        t = self.get_transport()
         tree_a = self.make_branch_and_tree('branch_a')
         branch_a = tree_a.branch
         self.build_tree(['branch_a/a'])
@@ -272,8 +258,8 @@
         branch_b = branch.Branch.open('branch_b')
         self.assertEqual(parent, branch_b.get_parent())
         # test implicit --remember after resolving previous failure
-        uncommit(branch=tree_b.branch, tree=tree_b)
-        transport.delete('branch_b/d')
+        uncommit.uncommit(branch=branch_b, tree=tree_b)
+        t.delete('branch_b/d')
         self.run_bzr('pull', working_dir='branch_b')
         branch_b = branch.Branch.open('branch_b')
         self.assertEqual(branch_b.get_parent(), parent)
@@ -287,26 +273,22 @@
         from bzrlib.testament import Testament
         # Build up 2 trees and prepare for a pull
         tree_a = self.make_branch_and_tree('branch_a')
-        f = open('branch_a/a', 'wb')
-        f.write('hello')
-        f.close()
+        with open('branch_a/a', 'wb') as f:
+            f.write('hello')
         tree_a.add('a')
         tree_a.commit('message')
 
         tree_b = tree_a.bzrdir.sprout('branch_b').open_workingtree()
 
         # Make a change to 'a' that 'b' can pull
-        f = open('branch_a/a', 'wb')
-        f.write('hey there')
-        f.close()
+        with open('branch_a/a', 'wb') as f:
+            f.write('hey there')
         tree_a.commit('message')
 
         # Create the bundle for 'b' to pull
-        os.chdir('branch_a')
-        self.run_bzr('bundle ../branch_b -o ../bundle')
+        self.run_bzr('bundle ../branch_b -o ../bundle', working_dir='branch_a')
 
-        os.chdir('../branch_b')
-        out, err = self.run_bzr('pull ../bundle')
+        out, err = self.run_bzr('pull ../bundle', working_dir='branch_b')
         self.assertEqual(out,
                          'Now on revision 2.\n')
         self.assertEqual(err,
@@ -323,7 +305,7 @@
                              testament_b.as_text())
 
         # it is legal to attempt to pull an already-merged bundle
-        out, err = self.run_bzr('pull ../bundle')
+        out, err = self.run_bzr('pull ../bundle', working_dir='branch_b')
         self.assertEqual(err, '')
         self.assertEqual(out, 'No revisions or tags to pull.\n')
 
@@ -432,7 +414,7 @@
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(19, self.hpss_calls)
         self.assertLength(1, self.hpss_connections)
-        remote = Branch.open('stacked')
+        remote = branch.Branch.open('stacked')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
 
     def test_pull_cross_format_warning(self):
@@ -502,26 +484,25 @@
         a_tree = self.example_branch('a')
         b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
 
-        f = open(pathjoin('a', 'hello'),'wt')
-        f.write('fee')
-        f.close()
+        with open(osutils.pathjoin('a', 'hello'),'wt') as f:
+            f.write('fee')
         a_tree.commit('fee')
 
-        f = open(pathjoin('b', 'hello'),'wt')
-        f.write('fie')
-        f.close()
+        with open(osutils.pathjoin('b', 'hello'),'wt') as f:
+            f.write('fie')
 
         out,err=self.run_bzr(['pull','-d','b','a','--show-base'])
 
         # check for message here
-        self.assertEqual(err,
-                         ' M  hello\nText conflict in hello\n1 conflicts encountered.\n')
+        self.assertEqual(
+            err,
+            ' M  hello\nText conflict in hello\n1 conflicts encountered.\n')
 
         self.assertEqualDiff('<<<<<<< TREE\n'
                              'fie||||||| BASE-REVISION\n'
                              'foo=======\n'
                              'fee>>>>>>> MERGE-SOURCE\n',
-                             open(pathjoin('b', 'hello')).read())
+                             open(osutils.pathjoin('b', 'hello')).read())
 
     def test_pull_show_base_working_tree_only(self):
         """--show-base only allowed if there's a working tree
@@ -531,8 +512,8 @@
         self.make_branch('from')
         self.make_branch('to')
         out=self.run_bzr(['pull','-d','to','from','--show-base'],retcode=3)
-        self.assertEqual(out,
-                         ('','bzr: ERROR: Need working tree for --show-base.\n'))
+        self.assertEqual(
+            out, ('','bzr: ERROR: Need working tree for --show-base.\n'))
 
     def test_pull_tag_conflicts(self):
         """pulling tags with conflicts will change the exit code"""

=== modified file 'bzrlib/tests/blackbox/test_remerge.py'
--- bzrlib/tests/blackbox/test_remerge.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_remerge.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,98 +24,102 @@
 class TestRemerge(TestCaseWithTransport):
 
     def make_file(self, name, contents):
-        f = open(name, 'wb')
-        try:
+        with open(name, 'wb') as f:
             f.write(contents)
-        finally:
-            f.close()
 
     def create_conflicts(self):
         """Create a conflicted tree"""
         os.mkdir('base')
-        os.chdir('base')
-        self.make_file('hello', "hi world")
-        self.make_file('answer', "42")
-        self.run_bzr('init')
-        self.run_bzr('add')
-        self.run_bzr('commit -m base')
-        self.run_bzr('branch . ../other')
-        self.run_bzr('branch . ../this')
-        os.chdir('../other')
-        self.make_file('hello', "Hello.")
-        self.make_file('answer', "Is anyone there?")
-        self.run_bzr('commit -m other')
-        os.chdir('../this')
-        self.make_file('hello', "Hello, world")
-        self.run_bzr('mv answer question')
-        self.make_file('question', "What do you get when you multiply six"
-                                   "times nine?")
-        self.run_bzr('commit -m this')
+        self.make_file('base/hello', "hi world")
+        self.make_file('base/answer', "42")
+        self.run_bzr('init', working_dir='base')
+        self.run_bzr('add', working_dir='base')
+        self.run_bzr('commit -m base', working_dir='base')
+        self.run_bzr('branch base other')
+        self.run_bzr('branch base this')
+        self.make_file('other/hello', "Hello.")
+        self.make_file('other/answer', "Is anyone there?")
+        self.run_bzr('commit -m other', working_dir='other')
+        self.make_file('this/hello', "Hello, world")
+        self.run_bzr('mv answer question', working_dir='this')
+        self.make_file('this/question',
+                       "What do you get when you multiply sixtimes nine?")
+        self.run_bzr('commit -m this', working_dir='this')
 
     def test_remerge(self):
         """Remerge command works as expected"""
         self.create_conflicts()
-        self.run_bzr('merge ../other --show-base', retcode=1)
-        conflict_text = open('hello').read()
+        self.run_bzr('merge ../other --show-base',
+                     retcode=1, working_dir='this')
+        conflict_text = open('this/hello').read()
         self.assertTrue('|||||||' in conflict_text)
         self.assertTrue('hi world' in conflict_text)
 
-        self.run_bzr_error(['conflicts encountered'], 'remerge', retcode=1)
-        conflict_text = open('hello').read()
+        self.run_bzr_error(['conflicts encountered'], 'remerge',
+                           retcode=1, working_dir='this')
+        with open('this/hello') as f:
+            conflict_text = f.read()
         self.assertFalse('|||||||' in conflict_text)
         self.assertFalse('hi world' in conflict_text)
 
-        os.unlink('hello.OTHER')
-        os.unlink('question.OTHER')
+        os.unlink('this/hello.OTHER')
+        os.unlink('this/question.OTHER')
 
         self.run_bzr_error(['jello is not versioned'],
-                     'remerge jello --merge-type weave')
+                     'remerge jello --merge-type weave', working_dir='this')
         self.run_bzr_error(['conflicts encountered'],
                            'remerge hello --merge-type weave',
-                           retcode=1)
-
-        self.assertPathExists('hello.OTHER')
-        self.assertPathDoesNotExist('question.OTHER')
-
-        file_id = self.run_bzr('file-id hello')[0]
+                           retcode=1, working_dir='this')
+
+        self.assertPathExists('this/hello.OTHER')
+        self.assertPathDoesNotExist('this/question.OTHER')
+
+        file_id = self.run_bzr('file-id hello', working_dir='this')[0]
         self.run_bzr_error(['hello.THIS is not versioned'],
-                           'file-id hello.THIS')
+                           'file-id hello.THIS', working_dir='this')
 
         self.run_bzr_error(['conflicts encountered'],
-                           'remerge --merge-type weave', retcode=1)
+                           'remerge --merge-type weave',
+                           retcode=1, working_dir='this')
 
-        self.assertPathExists('hello.OTHER')
-        self.assertTrue('hello.BASE')
+        self.assertPathExists('this/hello.OTHER')
+        self.assertTrue('this/hello.BASE')
+        with open('this/hello') as f:
+            conflict_text = f.read()
         self.assertFalse('|||||||' in conflict_text)
         self.assertFalse('hi world' in conflict_text)
 
         self.run_bzr_error(['Showing base is not supported.*Weave'],
-                           'remerge . --merge-type weave --show-base')
-        self.run_bzr_error(['Can\'t reprocess and show base'],
-                           'remerge . --show-base --reprocess')
+                           'remerge . --merge-type weave --show-base',
+                           working_dir='this')
+        self.run_bzr_error(["Can't reprocess and show base"],
+                           'remerge . --show-base --reprocess',
+                           working_dir='this')
         self.run_bzr_error(['conflicts encountered'],
                            'remerge . --merge-type weave --reprocess',
-                           retcode=1)
+                           retcode=1, working_dir='this')
         self.run_bzr_error(['conflicts encountered'],
                            'remerge hello --show-base',
-                           retcode=1)
+                           retcode=1, working_dir='this')
         self.run_bzr_error(['conflicts encountered'],
-                           'remerge hello --reprocess', retcode=1)
+                           'remerge hello --reprocess',
+                           retcode=1, working_dir='this')
 
-        self.run_bzr('resolve --all')
-        self.run_bzr('commit -m done')
+        self.run_bzr('resolve --all', working_dir='this')
+        self.run_bzr('commit -m done', working_dir='this')
 
         self.run_bzr_error(['remerge only works after normal merges',
                             'Not cherrypicking or multi-merges'],
-                           'remerge')
+                           'remerge', working_dir='this')
 
     def test_conflicts(self):
         self.create_conflicts()
-        self.run_bzr('merge ../other', retcode=1)
-        wt = WorkingTree.open('.')
-        self.assertEqual(2, len(wt.conflicts()))
-        self.run_bzr('remerge', retcode=1)
-        wt = WorkingTree.open('.')
-        self.assertEqual(2, len(wt.conflicts()))
-        self.run_bzr('remerge hello', retcode=1)
+        self.run_bzr('merge ../other', retcode=1, working_dir='this')
+        wt = WorkingTree.open('this')
+        self.assertEqual(2, len(wt.conflicts()))
+        self.run_bzr('remerge', retcode=1, working_dir='this')
+        wt = WorkingTree.open('this')
+        self.assertEqual(2, len(wt.conflicts()))
+        self.run_bzr('remerge hello', retcode=1, working_dir='this')
+        wt = WorkingTree.open('this')
         self.assertEqual(2, len(wt.conflicts()))

=== modified file 'bzrlib/tests/blackbox/test_remove_tree.py'
--- bzrlib/tests/blackbox/test_remove_tree.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_remove_tree.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,9 +36,8 @@
     # Success modes
 
     def test_remove_tree_original_branch(self):
-        os.chdir('branch1')
-        self.run_bzr('remove-tree')
-        self.assertPathDoesNotExist('foo')
+        self.run_bzr('remove-tree', working_dir='branch1')
+        self.assertPathDoesNotExist('branch1/foo')
 
     def test_remove_tree_original_branch_explicit(self):
         self.run_bzr('remove-tree branch1')
@@ -53,9 +52,8 @@
     def test_remove_tree_sprouted_branch(self):
         self.tree.bzrdir.sprout('branch2')
         self.assertPathExists('branch2/foo')
-        os.chdir('branch2')
-        self.run_bzr('remove-tree')
-        self.assertPathDoesNotExist('foo')
+        self.run_bzr('remove-tree', working_dir='branch2')
+        self.assertPathDoesNotExist('branch2/foo')
 
     def test_remove_tree_sprouted_branch_explicit(self):
         self.tree.bzrdir.sprout('branch2')
@@ -66,10 +64,8 @@
     def test_remove_tree_checkout(self):
         self.tree.branch.create_checkout('branch2', lightweight=False)
         self.assertPathExists('branch2/foo')
-        os.chdir('branch2')
-        self.run_bzr('remove-tree')
-        self.assertPathDoesNotExist('foo')
-        os.chdir('..')
+        self.run_bzr('remove-tree', working_dir='branch2')
+        self.assertPathDoesNotExist('branch2/foo')
         self.assertPathExists('branch1/foo')
 
     def test_remove_tree_checkout_explicit(self):
@@ -84,12 +80,10 @@
     def test_remove_tree_lightweight_checkout(self):
         self.tree.branch.create_checkout('branch2', lightweight=True)
         self.assertPathExists('branch2/foo')
-        os.chdir('branch2')
         output = self.run_bzr_error(
             ["You cannot remove the working tree from a lightweight checkout"],
-            'remove-tree', retcode=3)
-        self.assertPathExists('foo')
-        os.chdir('..')
+            'remove-tree', retcode=3, working_dir='branch2')
+        self.assertPathExists('branch2/foo')
         self.assertPathExists('branch1/foo')
 
     def test_remove_tree_lightweight_checkout_explicit(self):
@@ -103,9 +97,8 @@
 
     def test_remove_tree_empty_dir(self):
         os.mkdir('branch2')
-        os.chdir('branch2')
-        output = self.run_bzr_error(["Not a branch"],
-                                    'remove-tree', retcode=3)
+        output = self.run_bzr_error(
+            ["Not a branch"], 'remove-tree', retcode=3, working_dir='branch2')
 
     def test_remove_tree_repeatedly(self):
         self.run_bzr('remove-tree branch1')

=== modified file 'bzrlib/tests/blackbox/test_revision_history.py'
--- bzrlib/tests/blackbox/test_revision_history.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_revision_history.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,51 +14,53 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.branch import Branch
-
-
-class TestRevisionHistory(TestCaseWithTransport):
+from bzrlib import (
+    branch,
+    tests,
+    )
+
+
+class TestRevisionHistory(tests.TestCaseWithTransport):
 
     def _build_branch(self):
         # setup a standalone branch with three commits
         tree = self.make_branch_and_tree('test')
-        open('test/foo', 'wb').write('1111\n')
+        with open('test/foo', 'wb') as f:
+            f.write('1111\n')
         tree.add('foo')
         tree.commit('added foo',rev_id='revision_1')
-        open('test/foo', 'wb').write('2222\n')
+        with open('test/foo', 'wb')as f:
+            f.write('2222\n')
         tree.commit('updated foo',rev_id='revision_2')
-        open('test/foo', 'wb').write('3333\n')
+        with open('test/foo', 'wb')as f:
+            f.write('3333\n')
         tree.commit('updated foo again',rev_id='revision_3')
         return tree
 
-    def _check_revision_history(self, location=''):
-        rh = self.run_bzr(['revision-history', location])[0]
+    def _check_revision_history(self, location='', working_dir=None):
+        rh = self.run_bzr(['revision-history', location],
+                          working_dir=working_dir)[0]
         self.assertEqual(rh, 'revision_1\nrevision_2\nrevision_3\n')
 
     def test_revision_history(self):
-        """Tests 'revision_history' command"""
+        """No location"""
         self._build_branch()
-        os.chdir('test')
-        self._check_revision_history()
+        self._check_revision_history(working_dir='test')
 
     def test_revision_history_with_location(self):
-        """Tests 'revision_history' command with a specified location."""
+        """With a specified location."""
         self._build_branch()
         self._check_revision_history('test')
 
     def test_revision_history_with_repo_branch(self):
-        """Tests 'revision_history' command with a location that is a
-        repository branch."""
+        """With a repository branch location."""
         self._build_branch()
         self.run_bzr('init-repo repo')
         self.run_bzr('branch test repo/test')
         self._check_revision_history('repo/test')
 
     def test_revision_history_with_checkout(self):
-        """Tests 'revision_history' command with a location that is a
-        checkout of a repository branch."""
+        """With a repository branch checkout location."""
         self._build_branch()
         self.run_bzr('init-repo repo')
         self.run_bzr('branch test repo/test')
@@ -66,8 +68,7 @@
         self._check_revision_history('test-checkout')
 
     def test_revision_history_with_lightweight_checkout(self):
-        """Tests 'revision_history' command with a location that is a
-        lightweight checkout of a repository branch."""
+        """With a repository branch lightweight checkout location."""
         self._build_branch()
         self.run_bzr('init-repo repo')
         self.run_bzr('branch test repo/test')

=== modified file 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/tests/blackbox/test_versioning.py	2011-11-24 17:34:23 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -67,11 +67,9 @@
         self.run_bzr(['mkdir', 'dir'])
         self.assert_(os.path.isdir('dir'))
 
-        os.chdir('dir')
         self.log('Run mkdir in subdir')
-        self.run_bzr(['mkdir', 'subdir'])
-        self.assert_(os.path.isdir('subdir'))
-        os.chdir('..')
+        self.run_bzr(['mkdir', 'subdir'], working_dir='dir')
+        self.assert_(os.path.isdir('dir/subdir'))
 
         wt = WorkingTree.open('.')
 
@@ -156,9 +154,9 @@
         self.assertEqual(get_text_by_path(new, 'a/one'), 'new contents')
         new.unlock()
 
-        os.chdir('a')
         # commit from here should do nothing
-        self.run_bzr(['commit', '.', '-m', 'commit subdir only', '--unchanged'])
+        self.run_bzr(['commit', '.', '-m', 'commit subdir only', '--unchanged'],
+                     working_dir='a')
         v3 = b.repository.revision_tree(b.get_rev_id(3))
         v3.lock_read()
         self.assertEqual(get_text_by_path(v3, 'b/two'), 'old contents')
@@ -167,7 +165,8 @@
         v3.unlock()
 
         # commit in subdirectory commits whole tree
-        self.run_bzr(['commit', '-m', 'commit whole tree from subdir'])
+        self.run_bzr(['commit', '-m', 'commit whole tree from subdir'],
+                     working_dir='a')
         v4 = b.repository.revision_tree(b.get_rev_id(4))
         v4.lock_read()
         self.assertEqual(get_text_by_path(v4, 'b/two'), 'new contents')

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2012-01-02 17:59:40 +0000
+++ bzrlib/tests/test_merge.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -83,17 +83,18 @@
         tip = wt1.commit('empty commit')
         wt2 = self.make_branch_and_tree('branch2')
         wt2.pull(wt1.branch)
-        file('branch1/foo', 'wb').write('foo')
-        file('branch1/bar', 'wb').write('bar')
+        with file('branch1/foo', 'wb') as f:
+            f.write('foo')
+        with file('branch1/bar', 'wb') as f:
+            f.write('bar')
         wt1.add('foo')
         wt1.add('bar')
         wt1.commit('add foobar')
-        os.chdir('branch2')
-        self.run_bzr('merge ../branch1/baz', retcode=3)
-        self.run_bzr('merge ../branch1/foo')
-        self.assertPathExists('foo')
-        self.assertPathDoesNotExist('bar')
-        wt2 = WorkingTree.open('.') # opens branch2
+        self.run_bzr('merge ../branch1/baz', retcode=3, working_dir='branch2')
+        self.run_bzr('merge ../branch1/foo', working_dir='branch2')
+        self.assertPathExists('branch2/foo')
+        self.assertPathDoesNotExist('branch2/bar')
+        wt2 = WorkingTree.open('branch2')
         self.assertEqual([tip], wt2.get_parent_ids())
 
     def test_pending_with_null(self):

=== modified file 'bzrlib/tests/test_server.py'
--- bzrlib/tests/test_server.py	2012-01-05 15:57:44 +0000
+++ bzrlib/tests/test_server.py	2012-01-05 17:56:30 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010, 2011, 2012 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_uncommit.py'
--- bzrlib/tests/test_uncommit.py	2011-08-22 10:44:29 +0000
+++ bzrlib/tests/test_uncommit.py	2011-12-16 19:41:30 +0000
@@ -110,6 +110,27 @@
             "pointsatexisting": history[0]
             }, tree.branch.tags.get_tag_dict())
 
+    def test_uncommit_remove_tags_keeps_pending_merges(self):
+        tree, history = self.make_linear_tree()
+        copy = tree.bzrdir.sprout('copyoftree').open_workingtree()
+        copy.commit(message='merged', rev_id='merged')
+        tree.merge_from_branch(copy.branch)
+        tree.branch.tags.set_tag('pointsatmerged', 'merged')
+        history.append(tree.commit('merge'))
+        self.assertEquals('merged', tree.branch.tags.lookup_tag('pointsatmerged'))
+        self.assertEqual(history[2], tree.last_revision())
+        self.assertEqual((3, history[2]), tree.branch.last_revision_info())
+        tree.branch.tags.set_tag(u"pointsatexisting", history[1])
+        tree.branch.tags.set_tag(u"pointsatremoved", history[2])
+        uncommit.uncommit(tree.branch, tree=tree)
+        self.assertEqual(history[1], tree.last_revision())
+        self.assertEqual((2, history[1]), tree.branch.last_revision_info())
+        self.assertEquals([history[1], 'merged'], tree.get_parent_ids())
+        self.assertEqual({
+            "pointsatexisting": history[1],
+            "pointsatmerged": 'merged',
+            }, tree.branch.tags.get_tag_dict())
+
     def test_uncommit_keep_tags(self):
         tree, history = self.make_linear_tree()
         self.assertEqual(history[1], tree.last_revision())

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2011-12-18 20:46:39 +0000
+++ bzrlib/tests/test_urlutils.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -725,7 +725,7 @@
 
 
 class TestCwdToURL(TestCaseInTempDir):
-    """Test that local_path_to_url works base on the cwd"""
+    """Test that local_path_to_url works based on the cwd"""
 
     def test_dot(self):
         # This test will fail if getcwd is not ascii

=== modified file 'bzrlib/tests/test_whitebox.py'
--- bzrlib/tests/test_whitebox.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_whitebox.py	2012-01-05 13:02:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2011 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2012 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,15 +16,16 @@
 
 import os
 
+import bzrlib
 from bzrlib import (
+    errors,
     osutils,
+    tests,
     )
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.errors import PathNotChild
 from bzrlib.osutils import relpath, pathjoin, abspath, realpath
 
 
-class MoreTests(TestCaseWithTransport):
+class MoreTests(tests.TestCaseWithTransport):
 
     def test_relpath(self):
         """test for branch path lookups
@@ -33,52 +34,32 @@
         job: given a path (either relative to cwd or absolute), work out
         if it is inside a branch and return the path relative to the base.
         """
-        import tempfile
-
-        savedir = os.getcwdu()
         dtmp = osutils.mkdtemp()
+        self.addCleanup(osutils.rmtree, dtmp)
         # On Mac OSX, /tmp actually expands to /private/tmp
         dtmp = realpath(dtmp)
 
         def rp(p):
             return relpath(dtmp, p)
 
-        try:
-            # check paths inside dtmp while standing outside it
-            self.assertEqual(rp(pathjoin(dtmp, 'foo')), 'foo')
-
-            # root = nothing
-            self.assertEqual(rp(dtmp), '')
-
-            self.assertRaises(PathNotChild,
-                              rp,
-                              '/etc')
-
-            # now some near-miss operations -- note that
-            # os.path.commonprefix gets these wrong!
-            self.assertRaises(PathNotChild,
-                              rp,
-                              dtmp.rstrip('\\/') + '2')
-
-            self.assertRaises(PathNotChild,
-                              rp,
-                              dtmp.rstrip('\\/') + '2/foo')
-
-            # now operations based on relpath of files in current
-            # directory, or nearby
-            os.chdir(dtmp)
-
-            self.assertEqual(rp('foo/bar/quux'), 'foo/bar/quux')
-
-            self.assertEqual(rp('foo'), 'foo')
-
-            self.assertEqual(rp('./foo'), 'foo')
-
-            self.assertEqual(rp(abspath('foo')), 'foo')
-
-            self.assertRaises(PathNotChild,
-                              rp, '../foo')
-
-        finally:
-            os.chdir(savedir)
-            osutils.rmtree(dtmp)
+        # check paths inside dtmp while standing outside it
+        self.assertEqual('foo', rp(pathjoin(dtmp, 'foo')))
+
+        # root = nothing
+        self.assertEqual('', rp(dtmp))
+        self.assertRaises(errors.PathNotChild, rp, '/etc')
+
+        # now some near-miss operations -- note that
+        # os.path.commonprefix gets these wrong!
+        self.assertRaises(errors.PathNotChild, rp, dtmp.rstrip('\\/') + '2')
+        self.assertRaises(errors.PathNotChild, rp, dtmp.rstrip('\\/') + '2/foo')
+
+        # now operations based on relpath of files in current
+        # directory, or nearby
+
+        os.chdir(dtmp)
+        self.assertEqual('foo/bar/quux', rp('foo/bar/quux'))
+        self.assertEqual('foo', rp('foo'))
+        self.assertEqual('foo', rp('./foo'))
+        self.assertEqual('foo', rp(abspath('foo')))
+        self.assertRaises(errors.PathNotChild, rp, '../foo')

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2011-12-19 13:23:58 +0000
+++ bzrlib/uncommit.py	2012-01-05 16:03:11 +0000
@@ -28,17 +28,17 @@
 from bzrlib.errors import BoundBranchOutOfDate
 
 
-def remove_tags(branch, graph, old_tip, new_tip):
+def remove_tags(branch, graph, old_tip, parents):
     """Remove tags on revisions between old_tip and new_tip.
 
     :param branch: Branch to remove tags from
     :param graph: Graph object for branch repository
     :param old_tip: Old branch tip
-    :param new_tip: New branch tip
+    :param parents: New parents
     :return: Names of the removed tags
     """
     reverse_tags = branch.tags.get_reverse_tag_dict()
-    ancestors = graph.find_unique_ancestors(old_tip, [new_tip])
+    ancestors = graph.find_unique_ancestors(old_tip, parents)
     removed_tags = []
     for revid, tags in reverse_tags.iteritems():
         if not revid in ancestors:
@@ -130,15 +130,15 @@
                     hook_new_tip = None
                 hook(hook_local, hook_master, old_revno, old_tip, new_revno,
                      hook_new_tip)
-            if branch.supports_tags() and not keep_tags:
-                remove_tags(branch, graph, old_tip, new_revision_id)
+            if not _mod_revision.is_null(new_revision_id):
+                parents = [new_revision_id]
+            else:
+                parents = []
             if tree is not None:
-                if not _mod_revision.is_null(new_revision_id):
-                    parents = [new_revision_id]
-                else:
-                    parents = []
                 parents.extend(reversed(pending_merges))
                 tree.set_parent_ids(parents)
+            if branch.supports_tags() and not keep_tags:
+                remove_tags(branch, graph, old_tip, parents)
     finally:
         for item in reversed(unlockable):
             item.unlock()

=== removed directory 'bzrlib/util/effbot'
=== removed file 'bzrlib/util/effbot/__init__.py'
--- bzrlib/util/effbot/__init__.py	2011-12-18 15:28:38 +0000
+++ bzrlib/util/effbot/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,3 +0,0 @@
-# $Id: __init__.py 271 2004-10-09 10:50:59Z fredrik $
-from __future__ import absolute_import
-

=== removed directory 'bzrlib/util/effbot/org'
=== removed file 'bzrlib/util/effbot/org/__init__.py'
--- bzrlib/util/effbot/org/__init__.py	2011-12-18 15:28:38 +0000
+++ bzrlib/util/effbot/org/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,2 +0,0 @@
-# $Id: __init__.py 271 2004-10-09 10:50:59Z fredrik $
-from __future__ import absolute_import

=== removed file 'bzrlib/util/effbot/org/gzip_consumer.py'
--- bzrlib/util/effbot/org/gzip_consumer.py	2011-12-18 15:28:38 +0000
+++ bzrlib/util/effbot/org/gzip_consumer.py	1970-01-01 00:00:00 +0000
@@ -1,62 +0,0 @@
-# $Id: gzip_consumer.py 271 2004-10-09 10:50:59Z fredrik $
-# gzip consumer
-#
-# Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
-#
-
-from __future__ import absolute_import
-
-##
-# Consumer wrapper for GZIP streams.
-
-class GzipConsumer:
-
-    def __init__(self, consumer):
-        self.__consumer = consumer
-        self.__decoder = None
-        self.__data = ""
-
-    def __getattr__(self, key):
-        return getattr(self.__consumer, key)
-
-    def feed(self, data):
-        if self.__decoder is None:
-            # check if we have a full gzip header
-            data = self.__data + data
-            try:
-                i = 10
-                flag = ord(data[3])
-                if flag & 4: # extra
-                    x = ord(data[i]) + 256*ord(data[i+1])
-                    i = i + 2 + x
-                if flag & 8: # filename
-                    while ord(data[i]):
-                        i = i + 1
-                    i = i + 1
-                if flag & 16: # comment
-                    while ord(data[i]):
-                        i = i + 1
-                    i = i + 1
-                if flag & 2: # crc
-                    i = i + 2
-                if len(data) < i:
-                    raise IndexError("not enough data")
-                if data[:3] != "\x1f\x8b\x08":
-                    raise IOError("invalid gzip data")
-                data = data[i:]
-            except IndexError:
-                self.__data = data
-                return # need more data
-            import zlib
-            self.__data = ""
-            self.__decoder = zlib.decompressobj(-zlib.MAX_WBITS)
-        data = self.__decoder.decompress(data)
-        if data:
-            self.__consumer.feed(data)
-
-    def close(self):
-        if self.__decoder:
-            data = self.__decoder.flush()
-            if data:
-                self.__consumer.feed(data)
-        self.__consumer.close()

=== removed file 'bzrlib/util/effbot/org/http_client.py'
--- bzrlib/util/effbot/org/http_client.py	2011-12-18 15:28:38 +0000
+++ bzrlib/util/effbot/org/http_client.py	1970-01-01 00:00:00 +0000
@@ -1,270 +0,0 @@
-# $Id: http_client.py 271 2004-10-09 10:50:59Z fredrik $
-# a simple asynchronous http client (based on SimpleAsyncHTTP.py from
-# "Python Standard Library" by Fredrik Lundh, O'Reilly 2001)
-#
-# HTTP/1.1 and GZIP support added in January 2003 by Fredrik Lundh.
-#
-# changes:
-# 2004-08-26 fl   unified http callback
-# 2004-10-09 fl   factored out gzip_consumer support
-#
-# Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
-#
-
-from __future__ import absolute_import
-
-import asyncore
-import socket, string, time, sys
-import StringIO
-import mimetools, urlparse, urllib
-
-try:
-    from gzip_consumer import GzipConsumer
-except ImportError:
-    pass
-
-##
-# Close connection.   Request handlers can raise this exception to
-# indicate that the connection should be closed.
-
-class CloseConnection(Exception):
-    pass
-
-##
-# Redirect connection.  Request handlers can raise this exception to
-# indicate that the a new request should be issued.
-
-class Redirect(CloseConnection):
-    def __init__(self, location):
-        self.location = location
-
-##
-# Asynchronous HTTP/1.1 client.
-
-class async_http(asyncore.dispatcher_with_send):
-    # asynchronous http client
-
-    user_agent = "http_client.py 1.2 (http://effbot.org/zone)"
-    http_version = "1.1"
-
-    proxies = urllib.getproxies()
-
-    def __init__(self, uri, consumer, extra_headers=None):
-        asyncore.dispatcher_with_send.__init__(self)
-
-        # turn the uri into a valid request
-        scheme, host, path, params, query, fragment = urlparse.urlparse(uri)
-
-        # use origin host
-        self.host = host
-
-        # get proxy settings, if any
-        proxy = self.proxies.get(scheme)
-        if proxy:
-            scheme, host, x, x, x, x = urlparse.urlparse(proxy)
-
-        assert scheme == "http", "only supports HTTP requests (%s)" % scheme
-
-        if not path:
-            path = "/"
-        if params:
-            path = path + ";" + params
-        if query:
-            path = path + "?" + query
-        if proxy:
-            path = scheme + "://" + self.host + path
-
-        self.path = path
-
-        # get port number
-        try:
-            host, port = host.split(":", 1)
-            port = int(port)
-        except (TypeError, ValueError):
-            port = 80 # default port
-
-        self.consumer = consumer
-
-        self.status = None
-        self.header = None
-
-        self.bytes_in = 0
-        self.bytes_out = 0
-
-        self.content_type = None
-        self.content_length = None
-        self.content_encoding = None
-        self.transfer_encoding = None
-
-        self.data = ""
-
-        self.chunk_size = None
-
-        self.timestamp = time.time()
-
-        self.extra_headers = extra_headers
-
-        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
-        try:
-            self.connect((host, port))
-        except socket.error:
-            self.consumer.http(0, self, sys.exc_info())
-
-    def handle_connect(self):
-        # connection succeeded
-
-        request = [
-            "GET %s HTTP/%s" % (self.path, self.http_version),
-            "Host: %s" % self.host,
-            ]
-
-        if GzipConsumer:
-            request.append("Accept-Encoding: gzip")
-
-        if self.extra_headers:
-            request.extend(self.extra_headers)
-
-        # make sure to include a user agent
-        for header in request:
-            if string.lower(header).startswith("user-agent:"):
-                break
-        else:
-            request.append("User-Agent: %s" % self.user_agent)
-
-        request = string.join(request, "\r\n") + "\r\n\r\n"
-
-        self.send(request)
-
-        self.bytes_out = self.bytes_out + len(request)
-
-    def handle_expt(self):
-        # connection failed (windows); notify consumer
-
-        if sys.platform == "win32":
-            self.close()
-            self.consumer.http(0, self)
-
-    def handle_read(self):
-        # handle incoming data
-
-        data = self.recv(2048)
-
-        self.data = self.data + data
-        self.bytes_in = self.bytes_in + len(data)
-
-        while self.data:
-
-            if not self.header:
-                # check if we've seen a full header
-
-                header = self.data.split("\r\n\r\n", 1)
-                if len(header) <= 1:
-                    return
-                header, self.data = header
-
-                # parse header
-                fp = StringIO.StringIO(header)
-                self.status = fp.readline().split(" ", 2)
-                self.header = mimetools.Message(fp)
-
-                # get http headers
-                self.content_type = self.header.get("content-type")
-                try:
-                    self.content_length = int(
-                        self.header.get("content-length")
-                        )
-                except (ValueError, TypeError):
-                    self.content_length = None
-                self.transfer_encoding = self.header.get("transfer-encoding")
-                self.content_encoding = self.header.get("content-encoding")
-
-                if self.content_encoding == "gzip":
-                    # FIXME: report error if GzipConsumer is not available
-                    self.consumer = GzipConsumer(self.consumer)
-
-                try:
-                    self.consumer.http(1, self)
-                except Redirect, v:
-                    # redirect
-                    if v.location:
-                        do_request(
-                            v.location, self.consumer, self.extra_headers
-                            )
-                    self.close()
-                    return
-                except CloseConnection:
-                    self.close()
-                    return
-
-            if self.transfer_encoding == "chunked" and self.chunk_size is None:
-
-                # strip off leading whitespace
-                if self.data.startswith("\r\n"):
-                    self.data = self.data[2:]
-
-                chunk_size = self.data.split("\r\n", 1)
-                if len(chunk_size) <= 1:
-                    return
-                chunk_size, self.data = chunk_size
-
-                try:
-                    self.chunk_size = int(chunk_size, 16)
-                    if self.chunk_size <= 0:
-                        raise ValueError
-                except ValueError:
-                    return self.handle_close()
-
-            if not self.data:
-                return
-
-            data = self.data
-            self.data = ""
-
-            chunk_size = self.chunk_size or len(data)
-
-            if chunk_size < len(data):
-                self.data = data[chunk_size:]
-                data = data[:chunk_size]
-                self.chunk_size = None
-            else:
-                self.chunk_size = chunk_size - len(data)
-                if self.chunk_size <= 0:
-                    self.chunk_size = None
-
-            if data:
-                self.consumer.feed(data)
-
-            if self.content_length:
-                self.content_length -= chunk_size
-                if self.content_length <= 0:
-                    return self.handle_close()
-
-    def handle_close(self):
-        self.consumer.close()
-        self.close()
-
-    def handle_error(self):
-        self.consumer.http(0, self, sys.exc_info())
-        self.close()
-
-def do_request(uri, consumer, extra_headers=None):
-
-    return async_http(uri, consumer, extra_headers)
-
-if __name__ == "__main__":
-    class dummy_consumer:
-        def feed(self, data):
-            # print "feed", repr(data)
-            print "feed", repr(data[:20]), repr(data[-20:]), len(data)
-        def close(self):
-            print "close"
-        def http(self, ok, connection, **args):
-            print ok, connection, args
-            print "status", connection.status
-            print "header", connection.header
-    try:
-        url = sys.argv[1]
-    except IndexError:
-        url = "http://www.cnn.com/"
-    do_request(url, dummy_consumer())
-    asyncore.loop()

=== removed file 'bzrlib/util/effbot/org/http_manager.py'
--- bzrlib/util/effbot/org/http_manager.py	2011-12-18 15:28:38 +0000
+++ bzrlib/util/effbot/org/http_manager.py	1970-01-01 00:00:00 +0000
@@ -1,69 +0,0 @@
-# $Id: http_manager.py 270 2004-10-09 10:38:54Z fredrik $
-# effnews http
-#
-# manage a set of http clients
-#
-# Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
-#
-
-from __future__ import absolute_import
-
-import asyncore, time
-import http_client
-
-class http_manager:
-
-    max_connections = 8
-    max_size = 1000000
-    max_time = 60
-
-    def __init__(self):
-        self.queue = []
-
-    def request(self, uri, consumer, extra_headers=None):
-        self.queue.append((uri, consumer, extra_headers))
-
-    def priority_request(self, uri, consumer, extra_headers=None):
-        self.queue.insert(0, (uri, consumer, extra_headers))
-
-    def purge(self):
-        for channel in asyncore.socket_map.values():
-            channel.close()
-        del self.queue[:]
-
-    def prioritize(self, priority_uri):
-        i = 0
-        for uri, consumer, extra_headers in self.queue:
-            if uri == priority_uri:
-                del self.queue[i]
-                self.priority_request(uri, consumer, extra_headers)
-                return
-            i = i + 1
-
-    def poll(self, timeout=0.1):
-        # sanity checks
-        now = time.time()
-        for channel in asyncore.socket_map.values():
-            if channel.bytes_in > self.max_size:
-                channel.close() # too much data
-                try:
-                    channel.consumer.http(
-                        0, channel, ("HTTPManager", "too much data", None)
-                        )
-                except:
-                    pass
-            if channel.timestamp and now - channel.timestamp > self.max_time:
-                channel.close() # too slow
-                try:
-                    channel.consumer.http(
-                        0, channel, ("HTTPManager", "timeout", None)
-                        )
-                except:
-                    pass
-        # activate up to max_connections channels
-        while self.queue and len(asyncore.socket_map) < self.max_connections:
-            http_client.do_request(*self.queue.pop(0))
-        # keep the network running
-        asyncore.poll(timeout)
-        # return non-zero if we should keep on polling
-        return len(self.queue) or len(asyncore.socket_map)

=== removed directory 'bzrlib/util/elementtree'
=== removed file 'bzrlib/util/elementtree/ElementTree.py'
--- bzrlib/util/elementtree/ElementTree.py	2011-12-18 15:28:38 +0000
+++ bzrlib/util/elementtree/ElementTree.py	1970-01-01 00:00:00 +0000
@@ -1,1256 +0,0 @@
-#
-# ElementTree
-# $Id: ElementTree.py 2326 2005-03-17 07:45:21Z fredrik $
-#
-# light-weight XML support for Python 1.5.2 and later.
-#
-# history:
-# 2001-10-20 fl   created (from various sources)
-# 2001-11-01 fl   return root from parse method
-# 2002-02-16 fl   sort attributes in lexical order
-# 2002-04-06 fl   TreeBuilder refactoring, added PythonDoc markup
-# 2002-05-01 fl   finished TreeBuilder refactoring
-# 2002-07-14 fl   added basic namespace support to ElementTree.write
-# 2002-07-25 fl   added QName attribute support
-# 2002-10-20 fl   fixed encoding in write
-# 2002-11-24 fl   changed default encoding to ascii; fixed attribute encoding
-# 2002-11-27 fl   accept file objects or file names for parse/write
-# 2002-12-04 fl   moved XMLTreeBuilder back to this module
-# 2003-01-11 fl   fixed entity encoding glitch for us-ascii
-# 2003-02-13 fl   added XML literal factory
-# 2003-02-21 fl   added ProcessingInstruction/PI factory
-# 2003-05-11 fl   added tostring/fromstring helpers
-# 2003-05-26 fl   added ElementPath support
-# 2003-07-05 fl   added makeelement factory method
-# 2003-07-28 fl   added more well-known namespace prefixes
-# 2003-08-15 fl   fixed typo in ElementTree.findtext (Thomas Dartsch)
-# 2003-09-04 fl   fall back on emulator if ElementPath is not installed
-# 2003-10-31 fl   markup updates
-# 2003-11-15 fl   fixed nested namespace bug
-# 2004-03-28 fl   added XMLID helper
-# 2004-06-02 fl   added default support to findtext
-# 2004-06-08 fl   fixed encoding of non-ascii element/attribute names
-# 2004-08-23 fl   take advantage of post-2.1 expat features
-# 2005-02-01 fl   added iterparse implementation
-# 2005-03-02 fl   fixed iterparse support for pre-2.2 versions
-#
-# Copyright (c) 1999-2005 by Fredrik Lundh.  All rights reserved.
-#
-# fredrik@pythonware.com
-# http://www.pythonware.com
-#
-# --------------------------------------------------------------------
-# The ElementTree toolkit is
-#
-# Copyright (c) 1999-2005 by Fredrik Lundh
-#
-# By obtaining, using, and/or copying this software and/or its
-# associated documentation, you agree that you have read, understood,
-# and will comply with the following terms and conditions:
-#
-# Permission to use, copy, modify, and distribute this software and
-# its associated documentation for any purpose and without fee is
-# hereby granted, provided that the above copyright notice appears in
-# all copies, and that both that copyright notice and this permission
-# notice appear in supporting documentation, and that the name of
-# Secret Labs AB or the author not be used in advertising or publicity
-# pertaining to distribution of the software without specific, written
-# prior permission.
-#
-# SECRET LABS AB AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-# TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANT-
-# ABILITY AND FITNESS.  IN NO EVENT SHALL SECRET LABS AB OR THE AUTHOR
-# BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-# --------------------------------------------------------------------
-
-from __future__ import absolute_import
-
-__all__ = [
-    # public symbols
-    "Comment",
-    "dump",
-    "Element", "ElementTree",
-    "fromstring",
-    "iselement", "iterparse",
-    "parse",
-    "PI", "ProcessingInstruction",
-    "QName",
-    "SubElement",
-    "tostring",
-    "TreeBuilder",
-    "VERSION", "XML",
-    "XMLTreeBuilder",
-    ]
-
-##
-# The <b>Element</b> type is a flexible container object, designed to
-# store hierarchical data structures in memory. The type can be
-# described as a cross between a list and a dictionary.
-# <p>
-# Each element has a number of properties associated with it:
-# <ul>
-# <li>a <i>tag</i>. This is a string identifying what kind of data
-# this element represents (the element type, in other words).</li>
-# <li>a number of <i>attributes</i>, stored in a Python dictionary.</li>
-# <li>a <i>text</i> string.</li>
-# <li>an optional <i>tail</i> string.</li>
-# <li>a number of <i>child elements</i>, stored in a Python sequence</li>
-# </ul>
-#
-# To create an element instance, use the {@link #Element} or {@link
-# #SubElement} factory functions.
-# <p>
-# The {@link #ElementTree} class can be used to wrap an element
-# structure, and convert it from and to XML.
-##
-
-import string, sys, re
-
-class _SimpleElementPath:
-    # emulate pre-1.2 find/findtext/findall behaviour
-    def find(self, element, tag):
-        for elem in element:
-            if elem.tag == tag:
-                return elem
-        return None
-    def findtext(self, element, tag, default=None):
-        for elem in element:
-            if elem.tag == tag:
-                return elem.text or ""
-        return default
-    def findall(self, element, tag):
-        if tag[:3] == ".//":
-            return element.getiterator(tag[3:])
-        result = []
-        for elem in element:
-            if elem.tag == tag:
-                result.append(elem)
-        return result
-
-try:
-    import ElementPath
-except ImportError:
-    # FIXME: issue warning in this case?
-    ElementPath = _SimpleElementPath()
-
-# TODO: add support for custom namespace resolvers/default namespaces
-# TODO: add improved support for incremental parsing
-
-VERSION = "1.2.6"
-
-##
-# Internal element class.  This class defines the Element interface,
-# and provides a reference implementation of this interface.
-# <p>
-# You should not create instances of this class directly.  Use the
-# appropriate factory functions instead, such as {@link #Element}
-# and {@link #SubElement}.
-#
-# @see Element
-# @see SubElement
-# @see Comment
-# @see ProcessingInstruction
-
-class _ElementInterface:
-    # <tag attrib>text<child/>...</tag>tail
-
-    ##
-    # (Attribute) Element tag.
-
-    tag = None
-
-    ##
-    # (Attribute) Element attribute dictionary.  Where possible, use
-    # {@link #_ElementInterface.get},
-    # {@link #_ElementInterface.set},
-    # {@link #_ElementInterface.keys}, and
-    # {@link #_ElementInterface.items} to access
-    # element attributes.
-
-    attrib = None
-
-    ##
-    # (Attribute) Text before first subelement.  This is either a
-    # string or the value None, if there was no text.
-
-    text = None
-
-    ##
-    # (Attribute) Text after this element's end tag, but before the
-    # next sibling element's start tag.  This is either a string or
-    # the value None, if there was no text.
-
-    tail = None # text after end tag, if any
-
-    def __init__(self, tag, attrib):
-        self.tag = tag
-        self.attrib = attrib
-        self._children = []
-
-    def __repr__(self):
-        return "<Element %s at %x>" % (self.tag, id(self))
-
-    ##
-    # Creates a new element object of the same type as this element.
-    #
-    # @param tag Element tag.
-    # @param attrib Element attributes, given as a dictionary.
-    # @return A new element instance.
-
-    def makeelement(self, tag, attrib):
-        return Element(tag, attrib)
-
-    ##
-    # Returns the number of subelements.
-    #
-    # @return The number of subelements.
-
-    def __len__(self):
-        return len(self._children)
-
-    ##
-    # Returns the given subelement.
-    #
-    # @param index What subelement to return.
-    # @return The given subelement.
-    # @exception IndexError If the given element does not exist.
-
-    def __getitem__(self, index):
-        return self._children[index]
-
-    ##
-    # Replaces the given subelement.
-    #
-    # @param index What subelement to replace.
-    # @param element The new element value.
-    # @exception IndexError If the given element does not exist.
-    # @exception AssertionError If element is not a valid object.
-
-    def __setitem__(self, index, element):
-        assert iselement(element)
-        self._children[index] = element
-
-    ##
-    # Deletes the given subelement.
-    #
-    # @param index What subelement to delete.
-    # @exception IndexError If the given element does not exist.
-
-    def __delitem__(self, index):
-        del self._children[index]
-
-    ##
-    # Returns a list containing subelements in the given range.
-    #
-    # @param start The first subelement to return.
-    # @param stop The first subelement that shouldn't be returned.
-    # @return A sequence object containing subelements.
-
-    def __getslice__(self, start, stop):
-        return self._children[start:stop]
-
-    ##
-    # Replaces a number of subelements with elements from a sequence.
-    #
-    # @param start The first subelement to replace.
-    # @param stop The first subelement that shouldn't be replaced.
-    # @param elements A sequence object with zero or more elements.
-    # @exception AssertionError If a sequence member is not a valid object.
-
-    def __setslice__(self, start, stop, elements):
-        for element in elements:
-            assert iselement(element)
-        self._children[start:stop] = list(elements)
-
-    ##
-    # Deletes a number of subelements.
-    #
-    # @param start The first subelement to delete.
-    # @param stop The first subelement to leave in there.
-
-    def __delslice__(self, start, stop):
-        del self._children[start:stop]
-
-    ##
-    # Adds a subelement to the end of this element.
-    #
-    # @param element The element to add.
-    # @exception AssertionError If a sequence member is not a valid object.
-
-    def append(self, element):
-        assert iselement(element)
-        self._children.append(element)
-
-    ##
-    # Inserts a subelement at the given position in this element.
-    #
-    # @param index Where to insert the new subelement.
-    # @exception AssertionError If the element is not a valid object.
-
-    def insert(self, index, element):
-        assert iselement(element)
-        self._children.insert(index, element)
-
-    ##
-    # Removes a matching subelement.  Unlike the <b>find</b> methods,
-    # this method compares elements based on identity, not on tag
-    # value or contents.
-    #
-    # @param element What element to remove.
-    # @exception ValueError If a matching element could not be found.
-    # @exception AssertionError If the element is not a valid object.
-
-    def remove(self, element):
-        assert iselement(element)
-        self._children.remove(element)
-
-    ##
-    # Returns all subelements.  The elements are returned in document
-    # order.
-    #
-    # @return A list of subelements.
-    # @defreturn list of Element instances
-
-    def getchildren(self):
-        return self._children
-
-    ##
-    # Finds the first matching subelement, by tag name or path.
-    #
-    # @param path What element to look for.
-    # @return The first matching element, or None if no element was found.
-    # @defreturn Element or None
-
-    def find(self, path):
-        return ElementPath.find(self, path)
-
-    ##
-    # Finds text for the first matching subelement, by tag name or path.
-    #
-    # @param path What element to look for.
-    # @param default What to return if the element was not found.
-    # @return The text content of the first matching element, or the
-    #     default value no element was found.  Note that if the element
-    #     has is found, but has no text content, this method returns an
-    #     empty string.
-    # @defreturn string
-
-    def findtext(self, path, default=None):
-        return ElementPath.findtext(self, path, default)
-
-    ##
-    # Finds all matching subelements, by tag name or path.
-    #
-    # @param path What element to look for.
-    # @return A list or iterator containing all matching elements,
-    #    in document order.
-    # @defreturn list of Element instances
-
-    def findall(self, path):
-        return ElementPath.findall(self, path)
-
-    ##
-    # Resets an element.  This function removes all subelements, clears
-    # all attributes, and sets the text and tail attributes to None.
-
-    def clear(self):
-        self.attrib.clear()
-        self._children = []
-        self.text = self.tail = None
-
-    ##
-    # Gets an element attribute.
-    #
-    # @param key What attribute to look for.
-    # @param default What to return if the attribute was not found.
-    # @return The attribute value, or the default value, if the
-    #     attribute was not found.
-    # @defreturn string or None
-
-    def get(self, key, default=None):
-        return self.attrib.get(key, default)
-
-    ##
-    # Sets an element attribute.
-    #
-    # @param key What attribute to set.
-    # @param value The attribute value.
-
-    def set(self, key, value):
-        self.attrib[key] = value
-
-    ##
-    # Gets a list of attribute names.  The names are returned in an
-    # arbitrary order (just like for an ordinary Python dictionary).
-    #
-    # @return A list of element attribute names.
-    # @defreturn list of strings
-
-    def keys(self):
-        return self.attrib.keys()
-
-    ##
-    # Gets element attributes, as a sequence.  The attributes are
-    # returned in an arbitrary order.
-    #
-    # @return A list of (name, value) tuples for all attributes.
-    # @defreturn list of (string, string) tuples
-
-    def items(self):
-        return self.attrib.items()
-
-    ##
-    # Creates a tree iterator.  The iterator loops over this element
-    # and all subelements, in document order, and returns all elements
-    # with a matching tag.
-    # <p>
-    # If the tree structure is modified during iteration, the result
-    # is undefined.
-    #
-    # @param tag What tags to look for (default is to return all elements).
-    # @return A list or iterator containing all the matching elements.
-    # @defreturn list or iterator
-
-    def getiterator(self, tag=None):
-        nodes = []
-        if tag == "*":
-            tag = None
-        if tag is None or self.tag == tag:
-            nodes.append(self)
-        for node in self._children:
-            nodes.extend(node.getiterator(tag))
-        return nodes
-
-# compatibility
-_Element = _ElementInterface
-
-##
-# Element factory.  This function returns an object implementing the
-# standard Element interface.  The exact class or type of that object
-# is implementation dependent, but it will always be compatible with
-# the {@link #_ElementInterface} class in this module.
-# <p>
-# The element name, attribute names, and attribute values can be
-# either 8-bit ASCII strings or Unicode strings.
-#
-# @param tag The element name.
-# @param attrib An optional dictionary, containing element attributes.
-# @param **extra Additional attributes, given as keyword arguments.
-# @return An element instance.
-# @defreturn Element
-
-def Element(tag, attrib={}, **extra):
-    attrib = attrib.copy()
-    attrib.update(extra)
-    return _ElementInterface(tag, attrib)
-
-##
-# Subelement factory.  This function creates an element instance, and
-# appends it to an existing element.
-# <p>
-# The element name, attribute names, and attribute values can be
-# either 8-bit ASCII strings or Unicode strings.
-#
-# @param parent The parent element.
-# @param tag The subelement name.
-# @param attrib An optional dictionary, containing element attributes.
-# @param **extra Additional attributes, given as keyword arguments.
-# @return An element instance.
-# @defreturn Element
-
-def SubElement(parent, tag, attrib={}, **extra):
-    attrib = attrib.copy()
-    attrib.update(extra)
-    element = parent.makeelement(tag, attrib)
-    parent.append(element)
-    return element
-
-##
-# Comment element factory.  This factory function creates a special
-# element that will be serialized as an XML comment.
-# <p>
-# The comment string can be either an 8-bit ASCII string or a Unicode
-# string.
-#
-# @param text A string containing the comment string.
-# @return An element instance, representing a comment.
-# @defreturn Element
-
-def Comment(text=None):
-    element = Element(Comment)
-    element.text = text
-    return element
-
-##
-# PI element factory.  This factory function creates a special element
-# that will be serialized as an XML processing instruction.
-#
-# @param target A string containing the PI target.
-# @param text A string containing the PI contents, if any.
-# @return An element instance, representing a PI.
-# @defreturn Element
-
-def ProcessingInstruction(target, text=None):
-    element = Element(ProcessingInstruction)
-    element.text = target
-    if text:
-        element.text = element.text + " " + text
-    return element
-
-PI = ProcessingInstruction
-
-##
-# QName wrapper.  This can be used to wrap a QName attribute value, in
-# order to get proper namespace handling on output.
-#
-# @param text A string containing the QName value, in the form {uri}local,
-#     or, if the tag argument is given, the URI part of a QName.
-# @param tag Optional tag.  If given, the first argument is interpreted as
-#     an URI, and this argument is interpreted as a local name.
-# @return An opaque object, representing the QName.
-
-class QName:
-    def __init__(self, text_or_uri, tag=None):
-        if tag:
-            text_or_uri = "{%s}%s" % (text_or_uri, tag)
-        self.text = text_or_uri
-    def __str__(self):
-        return self.text
-    def __hash__(self):
-        return hash(self.text)
-    def __cmp__(self, other):
-        if isinstance(other, QName):
-            return cmp(self.text, other.text)
-        return cmp(self.text, other)
-
-##
-# ElementTree wrapper class.  This class represents an entire element
-# hierarchy, and adds some extra support for serialization to and from
-# standard XML.
-#
-# @param element Optional root element.
-# @keyparam file Optional file handle or name.  If given, the
-#     tree is initialized with the contents of this XML file.
-
-class ElementTree:
-
-    def __init__(self, element=None, file=None):
-        assert element is None or iselement(element)
-        self._root = element # first node
-        if file:
-            self.parse(file)
-
-    ##
-    # Gets the root element for this tree.
-    #
-    # @return An element instance.
-    # @defreturn Element
-
-    def getroot(self):
-        return self._root
-
-    ##
-    # Replaces the root element for this tree.  This discards the
-    # current contents of the tree, and replaces it with the given
-    # element.  Use with care.
-    #
-    # @param element An element instance.
-
-    def _setroot(self, element):
-        assert iselement(element)
-        self._root = element
-
-    ##
-    # Loads an external XML document into this element tree.
-    #
-    # @param source A file name or file object.
-    # @param parser An optional parser instance.  If not given, the
-    #     standard {@link XMLTreeBuilder} parser is used.
-    # @return The document root element.
-    # @defreturn Element
-
-    def parse(self, source, parser=None):
-        if getattr(source, "read", None) is None:
-            source = open(source, "rb")
-        if not parser:
-            parser = XMLTreeBuilder()
-        while 1:
-            data = source.read(32768)
-            if not data:
-                break
-            parser.feed(data)
-        self._root = parser.close()
-        return self._root
-
-    ##
-    # Creates a tree iterator for the root element.  The iterator loops
-    # over all elements in this tree, in document order.
-    #
-    # @param tag What tags to look for (default is to return all elements)
-    # @return An iterator.
-    # @defreturn iterator
-
-    def getiterator(self, tag=None):
-        assert self._root is not None
-        return self._root.getiterator(tag)
-
-    ##
-    # Finds the first toplevel element with given tag.
-    # Same as getroot().find(path).
-    #
-    # @param path What element to look for.
-    # @return The first matching element, or None if no element was found.
-    # @defreturn Element or None
-
-    def find(self, path):
-        assert self._root is not None
-        if path[:1] == "/":
-            path = "." + path
-        return self._root.find(path)
-
-    ##
-    # Finds the element text for the first toplevel element with given
-    # tag.  Same as getroot().findtext(path).
-    #
-    # @param path What toplevel element to look for.
-    # @param default What to return if the element was not found.
-    # @return The text content of the first matching element, or the
-    #     default value no element was found.  Note that if the element
-    #     has is found, but has no text content, this method returns an
-    #     empty string.
-    # @defreturn string
-
-    def findtext(self, path, default=None):
-        assert self._root is not None
-        if path[:1] == "/":
-            path = "." + path
-        return self._root.findtext(path, default)
-
-    ##
-    # Finds all toplevel elements with the given tag.
-    # Same as getroot().findall(path).
-    #
-    # @param path What element to look for.
-    # @return A list or iterator containing all matching elements,
-    #    in document order.
-    # @defreturn list of Element instances
-
-    def findall(self, path):
-        assert self._root is not None
-        if path[:1] == "/":
-            path = "." + path
-        return self._root.findall(path)
-
-    ##
-    # Writes the element tree to a file, as XML.
-    #
-    # @param file A file name, or a file object opened for writing.
-    # @param encoding Optional output encoding (default is US-ASCII).
-
-    def write(self, file, encoding="us-ascii"):
-        assert self._root is not None
-        if getattr(file, "write", None) is None:
-            file = open(file, "wb")
-        if not encoding:
-            encoding = "us-ascii"
-        elif encoding != "utf-8" and encoding != "us-ascii":
-            file.write("<?xml version='1.0' encoding='%s'?>\n" % encoding)
-        self._write(file, self._root, encoding, {})
-
-    def _write(self, file, node, encoding, namespaces):
-        # write XML to file
-        tag = node.tag
-        if tag is Comment:
-            file.write("<!-- %s -->" % _escape_cdata(node.text, encoding))
-        elif tag is ProcessingInstruction:
-            file.write("<?%s?>" % _escape_cdata(node.text, encoding))
-        else:
-            items = node.items()
-            xmlns_items = [] # new namespaces in this scope
-            try:
-                if isinstance(tag, QName) or tag[:1] == "{":
-                    tag, xmlns = fixtag(tag, namespaces)
-                    if xmlns: xmlns_items.append(xmlns)
-            except TypeError:
-                _raise_serialization_error(tag)
-            file.write("<" + _encode(tag, encoding))
-            if items or xmlns_items:
-                items.sort() # lexical order
-                for k, v in items:
-                    try:
-                        if isinstance(k, QName) or k[:1] == "{":
-                            k, xmlns = fixtag(k, namespaces)
-                            if xmlns: xmlns_items.append(xmlns)
-                    except TypeError:
-                        _raise_serialization_error(k)
-                    try:
-                        if isinstance(v, QName):
-                            v, xmlns = fixtag(v, namespaces)
-                            if xmlns: xmlns_items.append(xmlns)
-                    except TypeError:
-                        _raise_serialization_error(v)
-                    file.write(" %s=\"%s\"" % (_encode(k, encoding),
-                                               _escape_attrib(v, encoding)))
-                for k, v in xmlns_items:
-                    file.write(" %s=\"%s\"" % (_encode(k, encoding),
-                                               _escape_attrib(v, encoding)))
-            if node.text or len(node):
-                file.write(">")
-                if node.text:
-                    file.write(_escape_cdata(node.text, encoding))
-                for n in node:
-                    self._write(file, n, encoding, namespaces)
-                file.write("</" + _encode(tag, encoding) + ">")
-            else:
-                file.write(" />")
-            for k, v in xmlns_items:
-                del namespaces[v]
-        if node.tail:
-            file.write(_escape_cdata(node.tail, encoding))
-
-# --------------------------------------------------------------------
-# helpers
-
-##
-# Checks if an object appears to be a valid element object.
-#
-# @param An element instance.
-# @return A true value if this is an element object.
-# @defreturn flag
-
-def iselement(element):
-    # FIXME: not sure about this; might be a better idea to look
-    # for tag/attrib/text attributes
-    return isinstance(element, _ElementInterface) or (getattr(element, "tag", None) is not None)
-
-##
-# Writes an element tree or element structure to sys.stdout.  This
-# function should be used for debugging only.
-# <p>
-# The exact output format is implementation dependent.  In this
-# version, it's written as an ordinary XML file.
-#
-# @param elem An element tree or an individual element.
-
-def dump(elem):
-    # debugging
-    if not isinstance(elem, ElementTree):
-        elem = ElementTree(elem)
-    elem.write(sys.stdout)
-    tail = elem.getroot().tail
-    if not tail or tail[-1] != "\n":
-        sys.stdout.write("\n")
-
-def _encode(s, encoding):
-    try:
-        return s.encode(encoding)
-    except AttributeError:
-        return s # 1.5.2: assume the string uses the right encoding
-
-if sys.version[:3] == "1.5":
-    _escape = re.compile(r"[&<>\"\x80-\xff]+") # 1.5.2
-else:
-    _escape = re.compile(eval(r'u"[&<>\"\u0080-\uffff]+"'))
-
-_escape_map = {
-    "&": "&amp;",
-    "<": "&lt;",
-    ">": "&gt;",
-    '"': "&quot;",
-}
-
-_namespace_map = {
-    # "well-known" namespace prefixes
-    "http://www.w3.org/XML/1998/namespace": "xml",
-    "http://www.w3.org/1999/xhtml": "html",
-    "http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
-    "http://schemas.xmlsoap.org/wsdl/": "wsdl",
-}
-
-def _raise_serialization_error(text):
-    raise TypeError(
-        "cannot serialize %r (type %s)" % (text, type(text).__name__)
-        )
-
-def _encode_entity(text, pattern=_escape):
-    # map reserved and non-ascii characters to numerical entities
-    def escape_entities(m, map=_escape_map):
-        out = []
-        append = out.append
-        for char in m.group():
-            text = map.get(char)
-            if text is None:
-                text = "&#%d;" % ord(char)
-            append(text)
-        return string.join(out, "")
-    try:
-        return _encode(pattern.sub(escape_entities, text), "ascii")
-    except TypeError:
-        _raise_serialization_error(text)
-
-#
-# the following functions assume an ascii-compatible encoding
-# (or "utf-16")
-
-def _escape_cdata(text, encoding=None, replace=string.replace):
-    # escape character data
-    try:
-        if encoding:
-            try:
-                text = _encode(text, encoding)
-            except UnicodeError:
-                return _encode_entity(text)
-        text = replace(text, "&", "&amp;")
-        text = replace(text, "<", "&lt;")
-        text = replace(text, ">", "&gt;")
-        return text
-    except (TypeError, AttributeError):
-        _raise_serialization_error(text)
-
-def _escape_attrib(text, encoding=None, replace=string.replace):
-    # escape attribute value
-    try:
-        if encoding:
-            try:
-                text = _encode(text, encoding)
-            except UnicodeError:
-                return _encode_entity(text)
-        text = replace(text, "&", "&amp;")
-        text = replace(text, "'", "&apos;") # FIXME: overkill
-        text = replace(text, "\"", "&quot;")
-        text = replace(text, "<", "&lt;")
-        text = replace(text, ">", "&gt;")
-        return text
-    except (TypeError, AttributeError):
-        _raise_serialization_error(text)
-
-def fixtag(tag, namespaces):
-    # given a decorated tag (of the form {uri}tag), return prefixed
-    # tag and namespace declaration, if any
-    if isinstance(tag, QName):
-        tag = tag.text
-    namespace_uri, tag = string.split(tag[1:], "}", 1)
-    prefix = namespaces.get(namespace_uri)
-    if prefix is None:
-        prefix = _namespace_map.get(namespace_uri)
-        if prefix is None:
-            prefix = "ns%d" % len(namespaces)
-        namespaces[namespace_uri] = prefix
-        if prefix == "xml":
-            xmlns = None
-        else:
-            xmlns = ("xmlns:%s" % prefix, namespace_uri)
-    else:
-        xmlns = None
-    return "%s:%s" % (prefix, tag), xmlns
-
-##
-# Parses an XML document into an element tree.
-#
-# @param source A filename or file object containing XML data.
-# @param parser An optional parser instance.  If not given, the
-#     standard {@link XMLTreeBuilder} parser is used.
-# @return An ElementTree instance
-
-def parse(source, parser=None):
-    tree = ElementTree()
-    tree.parse(source, parser)
-    return tree
-
-##
-# Parses an XML document into an element tree incrementally, and reports
-# what's going on to the user.
-#
-# @param source A filename or file object containing XML data.
-# @param events A list of events to report back.  If omitted, only "end"
-#     events are reported.
-# @return A (event, elem) iterator.
-
-class iterparse:
-
-    def __init__(self, source, events=None):
-        if getattr(source, "read", None) is None:
-            source = open(source, "rb")
-        self._file = source
-        self._events = []
-        self._index = 0
-        self.root = self._root = None
-        self._parser = XMLTreeBuilder()
-        # wire up the parser for event reporting
-        parser = self._parser._parser
-        append = self._events.append
-        if events is None:
-            events = ["end"]
-        for event in events:
-            if event == "start":
-                try:
-                    parser.ordered_attributes = 1
-                    parser.specified_attributes = 1
-                    def handler(tag, attrib_in, event=event, append=append,
-                                start=self._parser._start_list):
-                        append((event, start(tag, attrib_in)))
-                    parser.StartElementHandler = handler
-                except AttributeError:
-                    def handler(tag, attrib_in, event=event, append=append,
-                                start=self._parser._start):
-                        append((event, start(tag, attrib_in)))
-                    parser.StartElementHandler = handler
-            elif event == "end":
-                def handler(tag, event=event, append=append,
-                            end=self._parser._end):
-                    append((event, end(tag)))
-                parser.EndElementHandler = handler
-            elif event == "start-ns":
-                def handler(prefix, uri, event=event, append=append):
-                    try:
-                        uri = _encode(uri, "ascii")
-                    except UnicodeError:
-                        pass
-                    append((event, (prefix or "", uri)))
-                parser.StartNamespaceDeclHandler = handler
-            elif event == "end-ns":
-                def handler(prefix, event=event, append=append):
-                    append((event, None))
-                parser.EndNamespaceDeclHandler = handler
-
-    def next(self):
-        while 1:
-            try:
-                item = self._events[self._index]
-            except IndexError:
-                if self._parser is None:
-                    self.root = self._root
-                    try:
-                        raise StopIteration
-                    except NameError:
-                        raise IndexError
-                # load event buffer
-                del self._events[:]
-                self._index = 0
-                data = self._file.read(16384)
-                if data:
-                    self._parser.feed(data)
-                else:
-                    self._root = self._parser.close()
-                    self._parser = None
-            else:
-                self._index = self._index + 1
-                return item
-
-    try:
-        iter
-        def __iter__(self):
-            return self
-    except NameError:
-        def __getitem__(self, index):
-            return self.next()
-
-##
-# Parses an XML document from a string constant.  This function can
-# be used to embed "XML literals" in Python code.
-#
-# @param source A string containing XML data.
-# @return An Element instance.
-# @defreturn Element
-
-def XML(text):
-    parser = XMLTreeBuilder()
-    parser.feed(text)
-    return parser.close()
-
-##
-# Parses an XML document from a string constant, and also returns
-# a dictionary which maps from element id:s to elements.
-#
-# @param source A string containing XML data.
-# @return A tuple containing an Element instance and a dictionary.
-# @defreturn (Element, dictionary)
-
-def XMLID(text):
-    parser = XMLTreeBuilder()
-    parser.feed(text)
-    tree = parser.close()
-    ids = {}
-    for elem in tree.getiterator():
-        id = elem.get("id")
-        if id:
-            ids[id] = elem
-    return tree, ids
-
-##
-# Parses an XML document from a string constant.  Same as {@link #XML}.
-#
-# @def fromstring(text)
-# @param source A string containing XML data.
-# @return An Element instance.
-# @defreturn Element
-
-fromstring = XML
-
-##
-# Generates a string representation of an XML element, including all
-# subelements.
-#
-# @param element An Element instance.
-# @return An encoded string containing the XML data.
-# @defreturn string
-
-def tostring(element, encoding=None):
-    class dummy:
-        pass
-    data = []
-    file = dummy()
-    file.write = data.append
-    ElementTree(element).write(file, encoding)
-    return string.join(data, "")
-
-##
-# Generic element structure builder.  This builder converts a sequence
-# of {@link #TreeBuilder.start}, {@link #TreeBuilder.data}, and {@link
-# #TreeBuilder.end} method calls to a well-formed element structure.
-# <p>
-# You can use this class to build an element structure using a custom XML
-# parser, or a parser for some other XML-like format.
-#
-# @param element_factory Optional element factory.  This factory
-#    is called to create new Element instances, as necessary.
-
-class TreeBuilder:
-
-    def __init__(self, element_factory=None):
-        self._data = [] # data collector
-        self._elem = [] # element stack
-        self._last = None # last element
-        self._tail = None # true if we're after an end tag
-        if element_factory is None:
-            element_factory = _ElementInterface
-        self._factory = element_factory
-
-    ##
-    # Flushes the parser buffers, and returns the toplevel documen
-    # element.
-    #
-    # @return An Element instance.
-    # @defreturn Element
-
-    def close(self):
-        assert len(self._elem) == 0, "missing end tags"
-        assert self._last is not None, "missing toplevel element"
-        return self._last
-
-    def _flush(self):
-        if self._data:
-            if self._last is not None:
-                text = string.join(self._data, "")
-                if self._tail:
-                    assert self._last.tail is None, "internal error (tail)"
-                    self._last.tail = text
-                else:
-                    assert self._last.text is None, "internal error (text)"
-                    self._last.text = text
-            self._data = []
-
-    ##
-    # Adds text to the current element.
-    #
-    # @param data A string.  This should be either an 8-bit string
-    #    containing ASCII text, or a Unicode string.
-
-    def data(self, data):
-        self._data.append(data)
-
-    ##
-    # Opens a new element.
-    #
-    # @param tag The element name.
-    # @param attrib A dictionary containing element attributes.
-    # @return The opened element.
-    # @defreturn Element
-
-    def start(self, tag, attrs):
-        self._flush()
-        self._last = elem = self._factory(tag, attrs)
-        if self._elem:
-            self._elem[-1].append(elem)
-        self._elem.append(elem)
-        self._tail = 0
-        return elem
-
-    ##
-    # Closes the current element.
-    #
-    # @param tag The element name.
-    # @return The closed element.
-    # @defreturn Element
-
-    def end(self, tag):
-        self._flush()
-        self._last = self._elem.pop()
-        assert self._last.tag == tag,\
-               "end tag mismatch (expected %s, got %s)" % (
-                   self._last.tag, tag)
-        self._tail = 1
-        return self._last
-
-##
-# Element structure builder for XML source data, based on the
-# <b>expat</b> parser.
-#
-# @keyparam target Target object.  If omitted, the builder uses an
-#     instance of the standard {@link #TreeBuilder} class.
-# @keyparam html Predefine HTML entities.  This flag is not supported
-#     by the current implementation.
-# @see #ElementTree
-# @see #TreeBuilder
-
-class XMLTreeBuilder:
-
-    def __init__(self, html=0, target=None):
-        try:
-            from xml.parsers import expat
-        except ImportError:
-            raise ImportError(
-                "No module named expat; use SimpleXMLTreeBuilder instead"
-                )
-        self._parser = parser = expat.ParserCreate(None, "}")
-        if target is None:
-            target = TreeBuilder()
-        self._target = target
-        self._names = {} # name memo cache
-        # callbacks
-        parser.DefaultHandlerExpand = self._default
-        parser.StartElementHandler = self._start
-        parser.EndElementHandler = self._end
-        parser.CharacterDataHandler = self._data
-        # let expat do the buffering, if supported
-        try:
-            self._parser.buffer_text = 1
-        except AttributeError:
-            pass
-        # use new-style attribute handling, if supported
-        try:
-            self._parser.ordered_attributes = 1
-            self._parser.specified_attributes = 1
-            parser.StartElementHandler = self._start_list
-        except AttributeError:
-            pass
-        encoding = None
-        if not parser.returns_unicode:
-            encoding = "utf-8"
-        # target.xml(encoding, None)
-        self._doctype = None
-        self.entity = {}
-
-    def _fixtext(self, text):
-        # convert text string to ascii, if possible
-        try:
-            return _encode(text, "ascii")
-        except UnicodeError:
-            return text
-
-    def _fixname(self, key):
-        # expand qname, and convert name string to ascii, if possible
-        try:
-            name = self._names[key]
-        except KeyError:
-            name = key
-            if "}" in name:
-                name = "{" + name
-            self._names[key] = name = self._fixtext(name)
-        return name
-
-    def _start(self, tag, attrib_in):
-        fixname = self._fixname
-        tag = fixname(tag)
-        attrib = {}
-        for key, value in attrib_in.items():
-            attrib[fixname(key)] = self._fixtext(value)
-        return self._target.start(tag, attrib)
-
-    def _start_list(self, tag, attrib_in):
-        fixname = self._fixname
-        tag = fixname(tag)
-        attrib = {}
-        if attrib_in:
-            for i in range(0, len(attrib_in), 2):
-                attrib[fixname(attrib_in[i])] = self._fixtext(attrib_in[i+1])
-        return self._target.start(tag, attrib)
-
-    def _data(self, text):
-        return self._target.data(self._fixtext(text))
-
-    def _end(self, tag):
-        return self._target.end(self._fixname(tag))
-
-    def _default(self, text):
-        prefix = text[:1]
-        if prefix == "&":
-            # deal with undefined entities
-            try:
-                self._target.data(self.entity[text[1:-1]])
-            except KeyError:
-                from xml.parsers import expat
-                raise expat.error(
-                    "undefined entity %s: line %d, column %d" %
-                    (text, self._parser.ErrorLineNumber,
-                    self._parser.ErrorColumnNumber)
-                    )
-        elif prefix == "<" and text[:9] == "<!DOCTYPE":
-            self._doctype = [] # inside a doctype declaration
-        elif self._doctype is not None:
-            # parse doctype contents
-            if prefix == ">":
-                self._doctype = None
-                return
-            text = string.strip(text)
-            if not text:
-                return
-            self._doctype.append(text)
-            n = len(self._doctype)
-            if n > 2:
-                type = self._doctype[1]
-                if type == "PUBLIC" and n == 4:
-                    name, type, pubid, system = self._doctype
-                elif type == "SYSTEM" and n == 3:
-                    name, type, system = self._doctype
-                    pubid = None
-                else:
-                    return
-                if pubid:
-                    pubid = pubid[1:-1]
-                self.doctype(name, pubid, system[1:-1])
-                self._doctype = None
-
-    ##
-    # Handles a doctype declaration.
-    #
-    # @param name Doctype name.
-    # @param pubid Public identifier.
-    # @param system System identifier.
-
-    def doctype(self, name, pubid, system):
-        pass
-
-    ##
-    # Feeds data to the parser.
-    #
-    # @param data Encoded data.
-
-    def feed(self, data):
-        self._parser.Parse(data, 0)
-
-    ##
-    # Finishes feeding data to the parser.
-    #
-    # @return An element structure.
-    # @defreturn Element
-
-    def close(self):
-        self._parser.Parse("", 1) # end of data
-        tree = self._target.close()
-        del self._target, self._parser # get rid of circular references
-        return tree

=== removed file 'bzrlib/util/elementtree/__init__.py'
--- bzrlib/util/elementtree/__init__.py	2011-12-18 15:28:38 +0000
+++ bzrlib/util/elementtree/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,32 +0,0 @@
-# $Id: __init__.py 1821 2004-06-03 16:57:49Z fredrik $
-# elementtree package
-
-# --------------------------------------------------------------------
-# The ElementTree toolkit is
-#
-# Copyright (c) 1999-2004 by Fredrik Lundh
-#
-# By obtaining, using, and/or copying this software and/or its
-# associated documentation, you agree that you have read, understood,
-# and will comply with the following terms and conditions:
-#
-# Permission to use, copy, modify, and distribute this software and
-# its associated documentation for any purpose and without fee is
-# hereby granted, provided that the above copyright notice appears in
-# all copies, and that both that copyright notice and this permission
-# notice appear in supporting documentation, and that the name of
-# Secret Labs AB or the author not be used in advertising or publicity
-# pertaining to distribution of the software without specific, written
-# prior permission.
-#
-# SECRET LABS AB AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-# TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANT-
-# ABILITY AND FITNESS.  IN NO EVENT SHALL SECRET LABS AB OR THE AUTHOR
-# BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-# --------------------------------------------------------------------
-
-from __future__ import absolute_import

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2011-12-19 13:23:58 +0000
+++ bzrlib/xml_serializer.py	2012-01-05 12:16:05 +0000
@@ -26,41 +26,32 @@
 
 import re
 
-from bzrlib.serializer import Serializer
-from bzrlib.trace import mutter
-
 try:
+    import xml.etree.cElementTree as elementtree
+    ParseError = getattr(elementtree, "ParseError", SyntaxError)
+except ImportError:
+    # Fall back to pure python implementation if C extention is unavailable
+    import xml.etree.ElementTree as elementtree
     try:
-        # it's in this package in python2.5
-        from xml.etree.cElementTree import (ElementTree, SubElement, Element,
-            XMLTreeBuilder, fromstring, tostring)
-        import xml.etree as elementtree
-        # Also import ElementTree module so monkey-patching below always works
-        import xml.etree.ElementTree
+        from xml.etree.ElementTree import ParseError
     except ImportError:
-        from cElementTree import (ElementTree, SubElement, Element,
-                                  XMLTreeBuilder, fromstring, tostring)
-        import elementtree.ElementTree
-    ParseError = SyntaxError
-except ImportError:
-    mutter('WARNING: using slower ElementTree; consider installing cElementTree'
-           " and make sure it's on your PYTHONPATH")
-    # this copy is shipped with bzr
-    from util.elementtree.ElementTree import (ElementTree, SubElement,
-                                              Element, XMLTreeBuilder,
-                                              fromstring, tostring)
-    import util.elementtree as elementtree
-    from xml.parsers.expat import ExpatError as ParseError
+        from xml.parsers.expat import ExpatError as ParseError
+
+(ElementTree, SubElement, Element, XMLTreeBuilder, fromstring, tostring) = (
+    elementtree.ElementTree, elementtree.SubElement, elementtree.Element,
+    elementtree.XMLTreeBuilder, elementtree.fromstring, elementtree.tostring)
+
 
 from bzrlib import (
     cache_utf8,
+    errors,
     inventory,
     lazy_regex,
-    errors,
+    serializer,
     )
 
 
-class XMLSerializer(Serializer):
+class XMLSerializer(serializer.Serializer):
     """Abstract XML object serialize/deserialize"""
 
     squashes_xml_invalid_characters = True

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2012-01-05 11:39:43 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2012-01-05 16:03:11 +0000
@@ -110,6 +110,9 @@
 * Stop altering ``sys.platform`` on OSX when initialising the locale.
   (Martin Packman, #570495)
 
+* Uncommit no longer removes tags if they are part of the working
+  trees pending merges. (Jelmer Vernooij, #905462)
+
 Documentation
 *************
 

=== removed file 'tools/http_client.py'
--- tools/http_client.py	2006-09-08 18:46:29 +0000
+++ tools/http_client.py	1970-01-01 00:00:00 +0000
@@ -1,501 +0,0 @@
-#! /usr/bin/python
-
-# $Id: http_client.py 271 2004-10-09 10:50:59Z fredrik $
-# a simple asynchronous http client (based on SimpleAsyncHTTP.py from
-# "Python Standard Library" by Fredrik Lundh, O'Reilly 2001)
-#
-# HTTP/1.1 and GZIP support added in January 2003 by Fredrik Lundh.
-#
-# changes:
-# 2004-08-26 fl   unified http callback
-# 2004-10-09 fl   factored out gzip_consumer support
-# 2005-07-08 mbp  experimental support for keepalive connections
-#
-# Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
-#
-
-
-
-"""async/pipelined http client
-
-Use
-===
-
-Users of this library pass in URLs they want to see, and consumer
-objects that will receive the results at some point in the future.
-Any number of requests may be queued up, and more may be added while
-the download is in progress.
-
-Requests can be both superscalar and superpipelined.  That is to say,
-for each server there can be multiple sockets open, and each socket
-may have more than one request in flight.
-
-Design
-======
-
-There is a single DownloadManager, and a connection object for each
-open socket.
-
-Request/consumer pairs are maintained in queues.  Each connection has
-a list of transmitted requests whose response has not yet been
-received.  There is also a per-server list of requests that have not
-yet been submitted.
-
-When a connection is ready to transmit a new request, it takes one
-from the unsubmitted list, sends the request, and adds the request to
-its unfulfilled list.  This should happen when the connection has
-space for more transmissions or when a new request is added by the
-user.  If the connection terminates with unfulfilled requests they are
-put back onto the unsubmitted list, to be retried elsewhere.
-
-Because responses come back precisely in order, the connection always
-knows what it should expect next: the response for the next
-unfulfilled request.
-"""
-
-# Note that (as of ubuntu python 2.4.1) every socket.connect() call
-# with a hostname does a remote DNS resolution, which is pretty sucky.
-# Shouldn't there be a cache in glibc?  We should probably cache the
-# address in, say, the DownloadManager.
-
-# TODO: A default consumer operation that writes the received data
-# into a file; by default the file is named the same as the last
-# component of the URL.
-
-# TODO: A utility function that is given a list of URLs, and downloads
-# them all parallel/pipelined.  If any fail, it raises an exception
-# (and discards the rest), or perhaps can be told to continue anyhow.
-# The content is written into temporary files.  It returns a list of
-# readable file objects.
-
-# TODO: If we try pipelined or keepalive and the connection drop out
-# then retry the request on a new connection; eventually we should perhaps
-# learn that a given host or network just won't allow keepalive.
-
-
-import asyncore
-import socket, string, time, sys
-import StringIO
-import mimetools, urlparse, urllib
-import logging
-
-logging.basicConfig(level=logging.DEBUG,
-                    format='%(asctime)s %(levelname)s %(message)s',
-                    filename='/tmp/http_client.log',
-                    filemode='w')
-
-logger = logging.getLogger('bzr.http_client')
-debug = logger.debug
-info = logger.info
-error = logger.error
-
-
-##
-# Close connection.   Request handlers can raise this exception to
-# indicate that the connection should be closed.
-
-class CloseConnection(Exception):
-    pass
-
-##
-# Redirect connection.  Request handlers can raise this exception to
-# indicate that the a new request should be issued.
-
-class Redirect(CloseConnection):
-    def __init__(self, location):
-        self.location = location
-
-
-class DownloadManager(object):
-    """Handles pipelined/overlapped downloads.
-
-    Pass in a series of URLs with handlers to receive the response.
-    This object will spread the requests over however many sockets
-    seem useful.
-
-    queued_requests
-        Requests not assigned to any channel
-
-    running_requests
-        Currently assigned to a channel
-    """
-    def __init__(self):
-        self.queued_requests = []
-        # self.channel = HttpChannel('localhost', 8000, self)
-        self.channels = []
-        self.try_pipelined = False
-        self.try_keepalive = False
-        self.max_channels = 5
-
-
-    def enqueue(self, url, consumer):
-        self.queued_requests.append((url, consumer))
-        self._wake_up_channel()
-
-
-    def _channel_closed(self, channel):
-        """Called by the channel when its socket closes.
-        """
-        self.channels.remove(channel)
-        if self.queued_requests:
-            # might recreate one
-            self._wake_up_channel()
-
-
-    def _make_channel(self):
-        # proxy2 203.17.154.69
-        # return HttpChannel('82.211.81.161', 80, self)         # bazaar-ng.org 
-        # return HttpChannel('203.17.154.69', 8080, self)
-        return HttpChannel('127.0.0.1', 8000, self)  # forwarded
-            
-
-    def _wake_up_channel(self):
-        """Try to wake up one channel to send the newly-added request.
-
-        There may be more than one request pending, and this may cause
-        more than one channel to take requests.  That's OK; some of
-        them may be frustrated.
-        """
-        from random import shuffle, choice
-        
-        # first, wake up any idle channels
-        done = False
-        for ch in self.channels:
-            if not ch.sent_requests:
-                ch.take_one()
-                done = True
-        if done:
-            debug("woke existing idle channel(s)")
-            return
-
-        if len(self.channels) < self.max_channels:
-            newch = self._make_channel()
-            self.channels.append(newch)
-            newch.take_one()
-            debug("created new channel")
-            return
-
-        if self.try_pipelined:
-            # ask existing channels to take it
-            debug("woke busy channel")
-            choice(self.channels).take_one()
-
-
-        # debug("request postponed until a channel's idle")
-        
-
-
-
-    def run(self):
-        """Run until all outstanding requests have been served."""
-        #while self.running_requests or self.queued_requests \
-        #          or not self.channel.is_idle():
-        #    asyncore.loop(count=1)
-        asyncore.loop()
-
-
-
-class Response(object):
-    """Holds in-flight response."""
-
-
-
-def _parse_response_http10(header):
-    from cStringIO import StringIO
-
-    fp = StringIO(header)
-    r = Response()
-
-    r.status = fp.readline().split(" ", 2)
-    r.headers = mimetools.Message(fp)
-
-    # we can only(?) expect to do keepalive if we got either a 
-    # content-length or chunked encoding; otherwise there's no way to know
-    # when the content ends apart from through the connection close
-    r.content_type = r.headers.get("content-type")
-    try:
-        r.content_length = int(r.headers.get("content-length"))
-    except (ValueError, TypeError):
-        r.content_length = None
-    debug("seen content length of %r" % r.content_length)
-
-    r.transfer_encoding = r.headers.get("transfer-encoding")
-    r.content_encoding = r.headers.get("content-encoding")
-    r.connection_reply = r.headers.get("connection")
-
-    # TODO: pass status code to consumer?
-
-    if r.transfer_encoding:
-        raise NotImplementedError()
-
-    if r.transfer_encoding:
-        raise NotImplementedError()
-
-    if int(r.status[1]) != 200:
-        debug("can't handle response status %r" % r.status)
-        raise NotImplementedError()
-
-    if r.content_length is None:
-        raise NotImplementedError()
-
-    if r.content_length == 0:
-        raise NotImplementedError()
-
-    r.content_remaining = r.content_length                
-
-    return r
-
-
-    
-    
-        
-
-
-class HttpChannel(asyncore.dispatcher_with_send):
-    """One http socket, pipelining if possible."""
-    # asynchronous http client
-
-    user_agent = "http_client.py 1.3ka (based on effbot)"
-
-    proxies = urllib.getproxies()
-
-    def __init__(self, ip_host, ip_port, manager):
-        asyncore.dispatcher_with_send.__init__(self)
-        self.manager = manager
-
-        # if a response header has been seen, this holds it
-        self.response = None
-        
-        self.data = ""
-
-        self.chunk_size = None
-
-        self.timestamp = time.time()
-
-        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
-        debug('connecting...')
-        self.connect((ip_host, ip_port))
-
-        # sent_requests holds (url, consumer) 
-        self.sent_requests = []
-
-        self._outbuf = ''
-
-
-    def __repr__(self):
-        return 'HttpChannel(local_port=%r)' % (self.getsockname(),)
-
-
-    def is_idle(self):
-        return (not self.sent_requests)
-
-
-    def handle_connect(self):
-        debug("connected")
-        self.take_one()
-
-
-    def take_one(self):
-        """Accept one request from the manager if possible."""
-        if self.manager.try_pipelined:
-            if len(self.sent_requests) > 4:
-                return
-        else:
-            if len(self.sent_requests) > 0:
-                return 
-        
-        try:
-            url, consumer = self.manager.queued_requests.pop(0)
-            debug('request accepted by channel')
-        except IndexError:
-            return
-        
-        # TODO: If there are too many already in flight, don't take one.
-        # TODO: If the socket's not writable (tx buffer full), don't take.
-        self._push_request_http10(url, consumer)
-
-
-
-    def _push_request_http10(self, url, consumer):
-        """Send a request, and add it to the outstanding queue."""
-        # TODO: check the url requested is appropriate for this connection
-
-        # TODO: If there are too many requests outstanding or (less likely) the 
-        # connection fails, queue it for later use.
-
-        # TODO: Keep track of requests that have been sent but not yet fulfilled,
-        # because we might need to retransmit them if the connection fails. (Or
-        # should the caller do that?)
-
-        request = self._form_request_http10(url)
-        debug('send request for %s from %r' % (url, self))
-
-        # dispatcher_with_send handles buffering the data until it can
-        # be written, and hooks handle_write.
-
-        self.send(request)
-
-        self.sent_requests.append((url, consumer))
-
-
-    def _form_request_http10(self, url):
-        # TODO: get right vhost name
-        request = [
-            "GET %s HTTP/1.0" % (url),
-            "Host: www.bazaar-ng.org",
-            ]
-
-        if self.manager.try_keepalive or self.manager.try_pipelined:
-            request.extend([
-                "Keep-Alive: 60", 
-                "Connection: keep-alive",
-                ])
-
-        # make sure to include a user agent
-        for header in request:
-            if string.lower(header).startswith("user-agent:"):
-                break
-        else:
-            request.append("User-Agent: %s" % self.user_agent)
-
-        return string.join(request, "\r\n") + "\r\n\r\n"
-
-
-    def handle_read(self):
-        # handle incoming data
-        data = self.recv(2048)
-
-        self.data = self.data + data
-
-        if len(data):
-            debug('got %d bytes from socket' % len(data))
-        else:
-            debug('server closed connection')
-
-        while self.data:
-            consumer = self.sent_requests[0][1]
-            if not self.response:
-                # do not have a full response header yet
-
-                # check if we've seen a full header
-                debug('getting header for %s' % self.sent_requests[0][0])
-
-                header = self.data.split("\r\n\r\n", 1)
-                if len(header) <= 1:
-                    return
-                header, self.data = header
-
-                self.response = _parse_response_http10(header)
-                self.content_remaining = self.response.content_length
-
-            if not self.data:
-                return
-
-            # we now know how many (more) content bytes we have, and how much
-            # is in the data buffer. there are two main possibilities:
-            # too much data, and some must be left behind containing the next
-            # response headers, or too little, or possibly just right
-
-            want = self.content_remaining
-            if want > 0:
-                got_data = self.data[:want]
-                self.data = self.data[want:]
-                
-                assert got_data
-
-                self.content_remaining -= len(got_data)
-
-                debug('pass back %d bytes of %s, %d remain'
-                      % (len(got_data),
-                         self.sent_requests[0][0],
-                         self.content_remaining))
-                consumer.feed(data)
-
-            if self.content_remaining == 0:
-                del self.sent_requests[0]
-
-                debug('content complete')
-                consumer.content_complete()
-                
-                # reset lots of things and try to get the next response header
-                if self.response.connection_reply == 'close':
-                    debug('server requested close')
-                    self.manager._channel_closed(self)
-                    self.close()
-                elif not self.manager.try_keepalive:
-                    debug('no keepalive for this socket')
-                    self.manager._channel_closed(self)
-                    self.close()
-                else:
-                    debug("ready for next header...")
-                    self.take_one()
-                self.response = None
-
-
-
-    def handle_close(self):
-        debug('async told us of close on %r' % self)
-        # if there are outstanding requests should probably reopen and 
-        # retransmit, but if we're not making any progress then give up
-        self.manager._channel_closed(self)
-        self.close()
-
-
-class DummyConsumer:
-    def __init__(self, url, pb):
-        self.url = url
-        self.outf = None
-        self._pb = pb
-
-    def feed(self, data):
-        # print "feed", repr(data)
-        # print "feed", repr(data[:20]), repr(data[-20:]), len(data)
-        if not self.outf:
-            base = self.url[self.url.rindex('/')+1:]
-            self.outf = file('/tmp/download/' + base, 'wb')
-        self.outf.write(data)
-
-    def error(self, err_info):
-        import traceback
-        error('error reported to consumer')
-        traceback.print_exception(err_info[0], err_info[1], err_info[2])
-        sys.exit(1)
-
-    def content_complete(self):
-        info('content complete from %s' % self.url)
-        self.outf.close()
-        self.outf = None
-        # using last_cnt is cheating
-        self._pb.update('downloading inventory',
-                        self._pb.last_cnt+1,
-                        self._pb.last_total)
-
-
-
-if __name__ == "__main__":
-    logging.basicConfig(level=logging.DEBUG)
-
-    mgr = DownloadManager()
-
-    from bzrlib.branch import Branch
-    from bzrlib.progress import ProgressBar
-
-    pb = ProgressBar()
-    revs = Branch('/home/mbp/work/bzr').revision_history()
-    pb.update('downloading inventories', 0, len(revs))
-
-    for rev in revs:
-        url = 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/inventory-store/' \
-              + rev + '.gz'
-        mgr.enqueue(url, DummyConsumer(url, pb))
-
-    mgr.run()
-    
-
-
-    
-#     for url in ['http://www.bazaar-ng.org/',
-#                 'http://www.bazaar-ng.org/tutorial.html',
-#                 'http://www.bazaar-ng.org/download.html',
-#                 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/revision-store/mbp@hope-20050415013653-3b3c9c3d33fae0a6.gz',
-#                 ]:

